﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Web;

using NHibernate.Linq;

using FootballPredictions;
using FootballPredictions.Mappings;
using SharpJags.Math;
using FootballPredictions.JAGS;

namespace FootballPredictions.DataHandling.JAGSOutputHandlers
{
    /// <summary>
    /// Saves the results of a simulation to the database (if the season simulation in constants is set to true).
    /// </summary>
    public class JAGSSeasonOutputHandler
    {
        private PredictiveModel Model { get; set; }
        private Simulation Simulation { get; set; }

        public JAGSSeasonOutputHandler(PredictiveModel model, Simulation simulation)
        {
            Model = model;
            Simulation = simulation;
            ExtractVariableInformation();
        }


        /// <summary>
        /// The output from JAGS (Coda files) is handled in this method.
        /// </summary>
        private void ExtractVariableInformation()
        {
            var goalsScored = new int[2, Model.NoMatches];
            var attack = new double[Model.NoTeams, Model.NoRounds];
            var defence = new double[Model.NoTeams, Model.NoRounds];
            
            var probabilities = new double[Model.NoMatches, 3];

            for (int i = 0; i < Constants.CHAINS; i++)
            {
                StreamReader fileIndex = new StreamReader(Constants.TEXT_FILES + "/JAGS/CODA" + i + "index.txt");
                StreamReader fileChain = new StreamReader(Constants.TEXT_FILES + "/JAGS/CODA" + i + "chain1.txt");

                var homeGoals = new int[Constants.SAMPLE_COUNT];
                var awayGoals = new int[Constants.SAMPLE_COUNT];
                var counter = 0;


                var indexLine = fileIndex.ReadLine();
                while (indexLine != null)
                {

                    var indexLineArray = indexLine.Split(new char[] { '[', ']', ',', ' ' });



                    for (int j = 0; j < Constants.SAMPLE_COUNT; j++)
                    {
                        var chainLine = fileChain.ReadLine();
                        var chainLineArray = chainLine.Split(' ');
                        if (indexLine.StartsWith("goalsScored"))
                        {
                            goalsScored[Convert.ToInt32(indexLineArray[1]) - 1, Convert.ToInt32(indexLineArray[2]) - 1]
                                += Convert.ToInt32(chainLineArray[2]);
                            if (counter < Constants.SAMPLE_COUNT)
                            {
                                homeGoals[j] = Convert.ToInt32(chainLineArray[2]);
                            }
                            else
                            {
                                awayGoals[j] = Convert.ToInt32(chainLineArray[2]);
                            }
                            counter++;
                        }
                        else if (indexLine.StartsWith("attack"))
                        {
                            attack[Convert.ToInt32(indexLineArray[1]) - 1, Convert.ToInt32(indexLineArray[2]) - 1]
                                += Convert.ToDouble(chainLineArray[2], CultureInfo.GetCultureInfo("en-GB"));
                        }
                        else if (indexLine.StartsWith("defence"))
                        {
                            defence[Convert.ToInt32(indexLineArray[1]) - 1, Convert.ToInt32(indexLineArray[2]) - 1]
                                += Convert.ToDouble(chainLineArray[2], CultureInfo.GetCultureInfo("en-GB"));
                        }
                    }

                    if (counter == Constants.SAMPLE_COUNT * 2)
                    {
                        var results = CalculateResultProbabilities(homeGoals, awayGoals);
                        probabilities[Convert.ToInt32(indexLineArray[2]) - 1, 0] += results[0];
                        probabilities[Convert.ToInt32(indexLineArray[2]) - 1, 1] += results[1];
                        probabilities[Convert.ToInt32(indexLineArray[2]) - 1, 2] += results[2];


                        homeGoals = new int[Constants.SAMPLE_COUNT];
                        awayGoals = new int[Constants.SAMPLE_COUNT];
                        counter = 0;
                    }

                    indexLine = fileIndex.ReadLine();
                }
            }

            for (int j = 0; j < Model.NoMatches; j++)
            {
                goalsScored[0, j] = (int)Math.Round((double)goalsScored[0, j] / (double)(Constants.SAMPLE_COUNT * Constants.CHAINS));
                goalsScored[1, j] = (int)Math.Round((double)goalsScored[1, j] / (double)(Constants.SAMPLE_COUNT * Constants.CHAINS));
                probabilities[j, 0] = probabilities[j, 0] / (double)Constants.CHAINS;
                probabilities[j, 1] = probabilities[j, 1] / (double)Constants.CHAINS;
                probabilities[j, 2] = probabilities[j, 2] / (double)Constants.CHAINS;
            }
            for (int j = 0; j < Model.NoTeams; j++)
            {
                for (int k = 0; k < Model.NoRounds; k++)
                {
                    attack[j, k] = attack[j, k] / (double)(Constants.SAMPLE_COUNT * Constants.CHAINS);
                    defence[j, k] = defence[j, k] / (double)(Constants.SAMPLE_COUNT * Constants.CHAINS);
                }
            }

            SavePredictionsToDatabase(goalsScored, attack, defence, probabilities);
        }

        private double[] CalculateResultProbabilities(int[] homeGoals, int[] awayGoals)
        {
            var home = 0.0;
            var draw = 0.0;
            var away = 0.0;
            for (int i = 0; i < Constants.SAMPLE_COUNT; i++)
            {
                if (homeGoals[i] > awayGoals[i]) home++;
                else if (homeGoals[i] == awayGoals[i]) draw++;
                else away++;
            }
            double[] values = { home / (double)Constants.SAMPLE_COUNT, draw / (double)Constants.SAMPLE_COUNT, away / (double)Constants.SAMPLE_COUNT };
            return values;
        }


        /// <summary>
        /// Create prediction objects and store them in the database
        /// </summary>
        private void SavePredictionsToDatabase(int[,] goalsScored, double[,] attack, double[,] defence, double[,] probabilities)
        {
            using (var session = NHibernateConfig.OpenSession())
            {
                var type = Model.GetType();
                
                var teamMatchCounter = new int[Model.NoTeams];
                // Saves the simulation
                
                var teamList = Model.Teams;
                var matches = Model.Matches;

                foreach (var match in matches)
                {
                    var matchIdx = matches.IndexOf(match);
                    var homeTeamIdx = teamList.IndexOf(match.HomeTeam);
                    var awayTeamIdx = teamList.IndexOf(match.AwayTeam);

                    object homeStrengthId;
                    object awayStrengthId;
                    object resultId;

                    

                    //Get result or create new result in db
                    using (var tx = session.BeginTransaction())
                    {
                        var homeGoals = goalsScored[0, matchIdx];
                        var awayGoals = goalsScored[1, matchIdx];

                        var result = from r in session.Query<Result>().ToList()
                                     where r.HomeGoals == homeGoals
                                     && r.AwayGoals == awayGoals
                                     select r;
                        if (result.Count<Result>() == 0)
                        {
                            var newResult = new Result()
                            {
                                HomeGoals = homeGoals,
                                AwayGoals = awayGoals
                            };
                            resultId = session.Save(newResult);
                            tx.Commit();
                        }
                        else
                        {
                            resultId = result.First<Result>().ResultId;
                        }
                    }

                    using (var tx = session.BeginTransaction())
                    {
                        var homeTeamStrength = new TeamStrength()
                        {
                            Attack = attack[homeTeamIdx, teamMatchCounter[homeTeamIdx]],
                            Defence = defence[homeTeamIdx, teamMatchCounter[homeTeamIdx]]
                        };
                        var awayTeamStrength = new TeamStrength()
                        {
                            Attack = attack[awayTeamIdx, teamMatchCounter[awayTeamIdx]],
                            Defence = defence[awayTeamIdx, teamMatchCounter[awayTeamIdx]]
                        };
                        teamMatchCounter[homeTeamIdx]++;
                        teamMatchCounter[awayTeamIdx]++;
                        homeStrengthId = session.Save(homeTeamStrength);
                        awayStrengthId = session.Save(awayTeamStrength);
                        tx.Commit();
                    }

                    using (var tx = session.BeginTransaction())
                    {
                        var prediction = new Prediction()
                        {
                            Match = session.Get<Match>(match.MatchId),
                            Simulation = session.Get<Simulation>(Simulation.SimulationId),

                            HomeProbability = probabilities[matchIdx, 0],
                            DrawProbability = probabilities[matchIdx, 1],
                            AwayProbability = probabilities[matchIdx, 2],

                            HomeStrength = session.Get<TeamStrength>(homeStrengthId),
                            AwayStrength = session.Get<TeamStrength>(awayStrengthId),
                            Result = session.Get<Result>(resultId),
                        };
                        session.Save(prediction);
                        tx.Commit();
                    }
                }
            }
        }
    }
}
