﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Web;

using MathNet.Numerics.Statistics;
using System.IO;
using System.Globalization;

namespace FootballPredictions.Evaluation
{
    /// <summary>
    /// Class for determining the autocorrelation between samples in a CodaChain file.
    /// </summary>
    public class Autocorrelation
    {
        string SampleType;
        int TeamNumber;
        int RoundNumber;
        int MaxTimeDistance;
        IList<Double> Samples = new List<Double>();
        IList<Double> AutoCorrValues = new List<Double>();

        public Autocorrelation(string sampleType, int teamNumber, int roundNumber, int maxTimeDistance)
        {
            SampleType = sampleType;
            TeamNumber = teamNumber;
            RoundNumber = roundNumber;
            MaxTimeDistance = maxTimeDistance;
        }

        public void CreatePlot()
        {
            ReadCodaChain();
            GenerateAutoCorrValues();
            PrintAutoCorrelationPlot();
        }

        /// <summary>
        /// Read CodaIndex file and find the requested parameter using SampleType and TeamNumber.
        /// Read CodaChain for the requested parameter.
        /// </summary>
        private void ReadCodaChain()
        {
            StreamReader fileIndex = new StreamReader(Constants.TEXT_FILES + "/JAGS/CODA" + 0 + "index.txt");
            var indexLine = "";
            var chainStartIndex = 0;
            while((indexLine = fileIndex.ReadLine()) != null)
            {
                if (indexLine.StartsWith(SampleType) && Convert.ToInt32(indexLine.Split('[', ',',']')[1]) == TeamNumber && Convert.ToInt32(indexLine.Split('[', ',', ']')[2]) == RoundNumber)
                {
                    chainStartIndex = Convert.ToInt32(indexLine.Split(' ')[1]);
                    break;
                }
            }
            fileIndex.Close();
            
            StreamReader fileChain = new StreamReader(Constants.TEXT_FILES + "/JAGS/CODA0chain1.txt");
            for (int i = 0; i < chainStartIndex; i++)
            {
                fileChain.ReadLine();
            }
            for (int i = 0; i < Constants.SAMPLES_AFTER_THINNING; i++)
            {
                Samples.Add(Convert.ToDouble(fileChain.ReadLine().Split(' ')[2], CultureInfo.GetCultureInfo("en-GB")));
            }
            fileChain.Close();
        }

        private void GenerateAutoCorrValues()
        {
            IList<Double> samples1;
            IList<Double> samples2;

            AutoCorrValues.Add(CalculateAutoCorrelation(Samples, Samples));

            for (int i = 1; i < MaxTimeDistance; i++)
            {
                samples1 = new List<Double>();
                samples2 = new List<Double>();
                for (int j = 0; j < Samples.Count(); j++)
                {
                    if (j < Samples.Count() - i)
                        samples1.Add(Samples[j]);
                    if (j >= i)
                        samples2.Add(Samples[j]);
                }
                AutoCorrValues.Add(CalculateAutoCorrelation(samples1, samples2));
            }

        }

        private double CalculateAutoCorrelation(IList<Double> samples1, IList<Double> samples2)
        {
            return Correlation.Pearson(samples1, samples2);
        }
        
        private void PrintAutoCorrelationPlot()
        {
            var prefix = "\\begin{figure} \n"
                        + "\\centering \n"
                        + "\\begin{tikzpicture} \n"
                        + "\\begin{axis} \n"
                        + "[legend pos=outer north east, \n"
                        + "xlabel=k, ylabel=Correlation coefficient] \n"
                        + "\\addplot [ybar, color=blue, fill] plot coordinates { \n";
            var coordinates = "";
            for (int i = 0; i < AutoCorrValues.Count(); i++)
            {
                coordinates += "("+i+","+AutoCorrValues[i].ToString(CultureInfo.GetCultureInfo("en-GB"))+") \n";
            }
            var suffix = "}; \n"
                        + "\\addplot [black, sharp plot, update limits = false] coordinates { \n"
                        + "(0,0) \n"
                        + "("+MaxTimeDistance+",0) \n"
                        + "}; \n"
                        + "\\legend{Autocorrelation} \n"
                        + "\\end{axis} \n"
                        + "\\end{tikzpicture}\\newline\\caption{Correlation, thinning "+Constants.THINNING+"}% \n"
                        + "\\label{pgfplots:<ENTER Caption>}% \n"
                        + "\\end{figure}";
            StreamWriter fileWriter = new StreamWriter(Constants.TEXT_FILES + "JAGS/" + "autocorrelation_"+Constants.SAMPLE_COUNT+"_"+Constants.THINNING+"_"+TeamNumber+"_"+RoundNumber+"_"+SampleType+".tex");
            fileWriter.Write(prefix + coordinates + suffix);
            fileWriter.Close();
        }
    }
}