﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Web;

namespace FootballPredictions.DataHandling.Plots
{
    /// <summary>
    /// Class for creating plots of monitored variables.
    /// NB! The parameters in Constants.cs are used here, so the CODA files in /textfiles/JAGS need to correspond with these values (sample count, chains, etc.).
    /// </summary>
    public class PlotMonitorValues
    {
        string SampleType;
        int TeamNumber;
        int RoundNumber;
        IList<IList<Double>> Samples = new List<IList<Double>>();

        public PlotMonitorValues(string sampleType, int teamNumber, int roundNumber)
        {
            SampleType = sampleType;
            TeamNumber = teamNumber;
            RoundNumber = roundNumber;
        }

        public void CreatePlot()
        {
            ReadCodaChain();
            PrintMonitorValuesPlot();
            PrintDistributionPlot();
        }

        /// <summary>
        /// Read CodaIndex file and find the requested parameter using SampleType and TeamNumber.
        /// Read CodaChain for the requested parameter.
        /// </summary>
        private void ReadCodaChain()
        {
            for (int i = 0; i < Constants.CHAINS; i++)
            {
                Samples.Add(new List<Double>());
                StreamReader fileIndex = new StreamReader(Constants.TEXT_FILES + "/JAGS/CODA" + i + "index.txt");
                var indexLine = "";
                var chainStartIndex = 0;
                while ((indexLine = fileIndex.ReadLine()) != null)
                {
                    if (indexLine.StartsWith(SampleType) && Convert.ToInt32(indexLine.Split('[', ',', ']')[1]) == TeamNumber && Convert.ToInt32(indexLine.Split('[', ',', ']')[2]) == RoundNumber)
                    {
                        chainStartIndex = Convert.ToInt32(indexLine.Split(' ')[1]);
                        break;
                    }
                }
                fileIndex.Close();

                StreamReader fileChain = new StreamReader(Constants.TEXT_FILES + "/JAGS/CODA" + i + "chain1.txt");
                for (int j = 0; j < chainStartIndex-1; j++)
                {
                    fileChain.ReadLine();
                }
                for (int j = 0; j < Constants.SAMPLES_AFTER_THINNING; j++)
                {
                    Samples[i].Add(Convert.ToDouble(fileChain.ReadLine().Split(' ')[2], CultureInfo.GetCultureInfo("en-GB")));
                }
                fileChain.Close();   
            }
        }
        
        private void PrintMonitorValuesPlot()
        {
            var colors = new string[] {"red", "green", "blue" };
            var prefix = "\\begin{figure} \n"
                        + "\\centering \n"
                        + "\\begin{tikzpicture} \n"
                        + "\\begin{axis} \n"
                        + "[legend pos=outer north east, \n"
                        + "xlabel=Samples, ylabel=Value] \n";
            var coordinates = "";
            for (int i = 0; i < Constants.CHAINS; i++)
            {
                coordinates += "\\addplot [color=" + colors[i] + "] plot coordinates { \n";
                for (int j = 0; j < Samples[i].Count(); j++)
                {
                    coordinates += "(" + j + "," + Samples[i][j].ToString(CultureInfo.GetCultureInfo("en-GB")) + ") \n";
                }
                coordinates += "}; \n";
            }
            var suffix =  "\\legend{Sample values} \n"
                        + "\\end{axis} \n"
                        + "\\end{tikzpicture}\\newline\\caption{Monitor " + SampleType + "values for team " + TeamNumber + "}% \n"
                        + "\\label{pgfplots:<ENTER Caption>}% \n"
                        + "\\end{figure}";
            StreamWriter fileWriter = new StreamWriter(Constants.TEXT_FILES + "JAGS/" + "monitorValuePlot_"+Constants.SAMPLE_COUNT+"_"+Constants.THINNING+"_"+TeamNumber+"_"+RoundNumber+"_"+SampleType+".tex");
            fileWriter.Write(prefix + coordinates + suffix);
            fileWriter.Close();
        }

        private void PrintDistributionPlot()
        {
            var prefix =  "\\begin{figure}\n"
                        + "\\centering\n"
                        + "\\begin{tikzpicture}\n"
                        + "\\begin{axis}[legend pos=outer north east, \n"
                        + "xlabel=Value, ylabel=Count] \n";
            var plotString = GenerateDistributionPlotCoordinates();
            var suffix =  "\\legend{Chain 1, Chain 2, Chain 3}\n"
                        + "\\end{axis}\n"
                        + "\\end{tikzpicture}\n"
                        + "\\caption{Distribution plot of sample values for an "+SampleType+" variable for "+Samples.Count+" chains}\n"
                        + "\\label{fig:<enter label here>}\n"
                        + "\\end{figure}";
            using (StreamWriter fileWriter = new StreamWriter(Constants.TEXT_FILES + "JAGS/" + "distributionPlot.tex"))
            {
                fileWriter.Write(prefix + plotString + suffix);
            }   
        }

        private string GenerateDistributionPlotCoordinates()
        {
            var output = "";
            var colors = new string[] { "red", "green", "blue" };
            var colorCount = 0;
            var max = 0.0;
            var min = 0.0;
            var intervals = 40;

            foreach (var samples in Samples)
            {
                if (samples.Max() > max) max = samples.Max();
                if (samples.Min() < min) min = samples.Min();
            }

            foreach (var samples in Samples)
            {
                var intervalSize = (max - min) / (double)intervals;
                var coordinates = new double[intervals];
                var bucketList = new Dictionary<double, int>(); //<coordinates, count>
                for (int i = 0; i < intervals; i++)
                {
                    coordinates[i] = min + i * intervalSize;
                    bucketList.Add(coordinates[i], 0);
                }
                foreach (var sample in samples)
                {
                    var bucket = 0.0;
                    for (int i = 0; i < intervals - 1; i++)
                    {
                        if (sample >= coordinates[i] && sample < coordinates[i + 1])
                        {
                            var lowBoundDist = Math.Abs(sample - coordinates[i]);
                            var highBoundDist = Math.Abs(sample - coordinates[i + 1]);
                            if (lowBoundDist < highBoundDist)
                            {
                                bucket = coordinates[i];
                            }
                            else
                            {
                                bucket = coordinates[i + 1];
                            }
                        }
                        else if (sample >= coordinates[intervals - 1])
                        {
                            bucket = coordinates[intervals - 1];
                        }
                    }
                    bucketList[bucket]++;
                }
                output += "\\addplot [smooth, color=" + colors[colorCount] + "] plot coordinates {\n";
                colorCount = (colorCount + 1) % colors.Length;
                foreach (var c in coordinates)
                {
                    if(bucketList[c] != 0)
                        output += "(" + c.ToString(CultureInfo.GetCultureInfo("en-GB")) + "," + bucketList[c] + ")\n";
                }
                output += "};\n";
            }
            return output;
        }
    }
}