﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

using FootballPredictions.Mappings;
using NHibernate.Linq;
using System.Globalization;

namespace FootballPredictions.DataHandling.Plots
{
    /// <summary>
    /// Plots a comparison of the odds and predictions.
    /// </summary>
    public class PredictionsAndOddsComparison
    {
        private IList<Prediction> Predictions;
        private IDictionary<int, Odds> Odds;
        private int StageId;
        private double[,] ComparisonArray;

        public PredictionsAndOddsComparison()
        { }

        public void Compare(string bookmakerID, int simulationId)
        {
            SetPredictionsBySimulationId(simulationId);
            Predictions.OrderBy(p => p.Match.DateAndTime);
            SetOddsByBookmakerId(bookmakerID, simulationId);
            ComparisonArray = new double[Predictions.Count(), 6];

            var matchCounter = 0;
            foreach (var p in Predictions)
            {
                using (var session = NHibernateConfig.OpenSession())
                {
                    var match = session.Get<Match>(p.Match.MatchId).MatchId;
                    ComparisonArray[matchCounter, 0] = p.HomeProbability;
                    //ComparisonArray[matchCounter, 1] = 1.0 / Odds[match].HomeOdds;
                    ComparisonArray[matchCounter, 1] = (1/Odds[match].HomeOdds) / ((1/Odds[match].HomeOdds) + (1/Odds[match].DrawOdds) + (1/Odds[match].AwayOdds));
                    ComparisonArray[matchCounter, 2] = p.DrawProbability;
                    //ComparisonArray[matchCounter, 3] = 1.0 / Odds[match].DrawOdds;
                    ComparisonArray[matchCounter, 3] = (1 / Odds[match].DrawOdds) / ((1 / Odds[match].HomeOdds) + (1 / Odds[match].DrawOdds) + (1 / Odds[match].AwayOdds));
                    ComparisonArray[matchCounter, 4] = p.AwayProbability;
                    //ComparisonArray[matchCounter, 5] = 1.0 / Odds[match].AwayOdds;
                    ComparisonArray[matchCounter, 5] = (1 / Odds[match].AwayOdds) / ((1 / Odds[match].HomeOdds) + (1 / Odds[match].DrawOdds) + (1 / Odds[match].AwayOdds));
                }
                matchCounter++;
            }
            WriteComparisonToTex(simulationId, bookmakerID);
        }

        /// <summary>
        /// Retrieve all the predictions of a given simulation.
        /// </summary>
        private void SetPredictionsBySimulationId(int simulationId)
        {
            using (var session = NHibernateConfig.OpenSession())
            {
                var predictions = from p in session.Query<Prediction>().ToList()
                                  where p.Simulation.SimulationId == simulationId
                                  select p;
                Predictions = predictions.ToList();
                StageId = predictions.First().Match.Stage.StageId;
            }
        }

        /// <summary>
        /// Retrieve all odds from a given bookmaker for a season.
        /// </summary>
        private void SetOddsByBookmakerId(string bookmakerId, int simulationId)
        {
            using (var session = NHibernateConfig.OpenSession())
            {
                var odds = from o in session.Query<Odds>().ToList()
                           where o.Bookmaker.BookmakerId == bookmakerId
                           && o.Match.Stage.StageId == StageId
                           select o;
                //foreach (var o in odds)
                //{
                //    Odds.Add(o.Match.MatchId, o);
                //}
                Odds = odds.ToDictionary(o => o.Match.MatchId);
            }
        }

        private void WriteComparisonToTex(int simulationId, string bookmakerId)
        {
            var output = "";
            var colorCount = 0;

            var prefix = "\\begin{figure} \n"
                        + "\\centering \n"
                        + "\\begin{tikzpicture} \n"
                        + "\\begin{axis}\n"
                        + "[legend pos=outer north east, \n"
                        + "xlabel=Match, ylabel=Probability]";

            var suffix = "\\legend{Probability, Odds} \n"
                            + "\\end{axis} \n"
                            + "\\end{tikzpicture}\\newline"
                            + "\\caption{<ENTER Figure Text>}% \n"
                            + "\\label{pgfplots:<ENTER Caption>}% \n"
                            + "\\end{figure}\n";

            output += prefix + "\n";

            for (int i = 0; i < 2; i++)
            {
                var colorAndMark = GetColorAndMark(colorCount++);
                output += "\\addplot[color=" + colorAndMark[0] + ",mark=" + colorAndMark[1] + "] coordinates {\n";
                for (int j = 0; j < Predictions.Count(); j++)
                {
                    output += "(" + j + "," + ComparisonArray[j, i].ToString(CultureInfo.GetCultureInfo("en-GB")) + ")\n";
                }
                output += "};\n";
            }
            output += suffix;

            output += prefix + "\n";

            for (int i = 2; i < 4; i++)
            {
                var colorAndMark = GetColorAndMark(colorCount++);
                output += "\\addplot[color=" + colorAndMark[0] + ",mark=" + colorAndMark[1] + "] coordinates {\n";
                for (int j = 0; j < Predictions.Count(); j++)
                {
                    output += "(" + j + "," + ComparisonArray[j, i].ToString(CultureInfo.GetCultureInfo("en-GB")) + ")\n";
                }
                output += "};\n";
            }
            output += suffix;

            output += prefix + "\n";

            for (int i = 4; i < 6; i++)
            {
                var colorAndMark = GetColorAndMark(colorCount++);
                output += "\\addplot[color=" + colorAndMark[0] + ",mark=" + colorAndMark[1] + "] coordinates {\n";
                for (int j = 0; j < Predictions.Count(); j++)
                {
                    output += "(" + j + "," + ComparisonArray[j, i].ToString(CultureInfo.GetCultureInfo("en-GB")) + ")\n";
                }
                output += "};\n";
            }
            output += suffix;
            using (System.IO.StreamWriter file = new System.IO.StreamWriter(
              Constants.TEXT_FILES + "JAGS/Comparison/"
              + simulationId + "_"
              + bookmakerId + "_"
              + "comparison.tex"))
            {
                file.Write(output);
            }
        }

        /// <summary>
        /// Generate new color and mark for new plot
        /// </summary>
        private string[] GetColorAndMark(int count)
        {
            string[] colors = new string[] { "blue", "orange", "blue", "orange", "blue", "orange" };
            string[] marks = new string[] { "x", "o" };

            return new string[] { colors[count % 6], marks[count % 2] };
        }
    }
}