﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Web;

using FootballPredictions.Mappings;


namespace FootballPredictions.DataHandling.PopulateDatabase
{
    /// <summary>
    /// Populates the database with infomration about all the matches, and relates them to the team's participating.
    /// </summary>
    public class PopulateTeamMatch
    {
        public int MatchId { get; set; }
        public int HomeTeamId { get; set; }
        public int AwayTeamId { get; set; }
        public TeamMatch currentTeamMatch { get; set; }

        /// <summary>
        /// Constructor. All match information files are read and handled here.
        /// </summary>
        public PopulateTeamMatch()
        {
            string pattern = "*.txt";
            DirectoryInfo root = new DirectoryInfo(@"" + Constants.TEXT_FILES + Constants.STAGE_ID + "/RawFiles");
            IEnumerable<FileInfo> desiredFiles = root.EnumerateFiles(pattern, SearchOption.AllDirectories);

            foreach (FileInfo fi in desiredFiles)
            {
                DocumentFormatter docFormat = new DocumentFormatter();
                System.IO.StreamReader formattedFile = new System.IO.StreamReader(docFormat.FormatMatchDocument(fi));

                string line;

                MatchId = Convert.ToInt32(Regex.Replace(fi.Name, "[^0-9]", ""));

                while ((line = formattedFile.ReadLine()) != null)
                {

                    if (line.StartsWith("MatchInfo"))
                    {
                        FirstLine(line);
                    }

                    else if (line.StartsWith("Team"))
                    {
                        TeamLine(line);
                    }

                    else if (line.StartsWith("Player"))
                    {
                        PlayerLine(line);
                    }

                    else if (line.StartsWith("home") || line.StartsWith("away"))
                    {
                        ProbableLineUpLine(line);
                    }
                }
            }
        }

        /// <summary>
        /// The first line of a match info file contains general match info.
        /// This method extracts general match info and stores it in the database.
        /// </summary>
        public void FirstLine(string line)
        {
            string[] lineTable = line.Split(',');
            HomeTeamId = Convert.ToInt32(lineTable[1]);
            AwayTeamId = Convert.ToInt32(lineTable[2]);
            var tempDate = lineTable[3];
            tempDate = tempDate.Replace("'", "");
            DateTime date = DateTime.ParseExact(tempDate, "MM/dd/yyyy HH:mm:ss", CultureInfo.InvariantCulture);
            string[] goals = lineTable[4].Replace(" ", "").Split(':');
            var homeGoals = Convert.ToInt32(goals[0].Replace("'", ""));
            var awayGoals = Convert.ToInt32(goals[1].Replace("'", ""));
            var result = 'D';
            if (homeGoals > awayGoals) result = 'H';
            else if (homeGoals < awayGoals) result = 'A';

            using (var session = NHibernateConfig.OpenSession())
            {
                using (var tx = session.BeginTransaction())
                {
                    var match = new Mappings.Match
                    {
                        MatchId = MatchId,
                        Stage = session.Get<Stage>(Constants.STAGE_ID),
                        HomeTeam = session.Get<Team>(HomeTeamId),
                        AwayTeam = session.Get<Team>(AwayTeamId),
                        HomeGoals = homeGoals,
                        AwayGoals = awayGoals,
                        Result = result,
                        DateAndTime = date
                    };
                    session.SaveOrUpdate(match);
                    tx.Commit();
                }
            }
        }

        /// <summary>
        /// Extracts team performance information and stores it in the database
        /// </summary>
        public void TeamLine(string line)
        {
            string[] teamInformation = line.Split(',');
            var teamId = Convert.ToInt32(teamInformation[1]);
            var rating = Convert.ToDouble(teamInformation[3], CultureInfo.InvariantCulture);
            var possessionPercentage = 0.0;
            IDictionary<string, int> attributes = new Dictionary<string, int>();

            for (int i = 4; i < teamInformation.Length; i = i + 2)
            {
                if (teamInformation[i].Equals("possession_percentage"))
                {
                    possessionPercentage = Convert.ToDouble(teamInformation[i + 1], CultureInfo.InvariantCulture);
                }
                else if (!attributes.ContainsKey(teamInformation[i]))
                {
                    attributes.Add(teamInformation[i], Convert.ToInt32(teamInformation[i + 1]));
                }

            }

            using (var session = NHibernateConfig.OpenSession())
            {
                using (var tx = session.BeginTransaction())
                {
                    var teamMatch = new TeamMatch
                    {
                        Team = session.Get<Team>(teamId),
                        Match = session.Get<FootballPredictions.Mappings.Match>(MatchId),
                        
                        Rating = rating,
                        PossessionPercentage = possessionPercentage,
                        
                        Goals = checkValue(attributes, "goals"),
                        OwnGoals = checkValue(attributes, "own_goals"),
                        OwnGoalAccrued = checkValue(attributes, "own_goal_accrued"),
                        AttPenGoal = checkValue(attributes, "att_pen_goal"),

                        Touches = checkValue(attributes, "touches"),
                        UnsuccessfulTouch = checkValue(attributes, "unsuccessful_touch"),
                        GoalAssist = checkValue(attributes, "goal_assist"),
                        TotalPass = checkValue(attributes, "total_pass"),
                        AccuratePass = checkValue(attributes, "accurate_pass"),
                        TotalThroughBall = checkValue(attributes, "total_through_ball"),
                        AccurateThroughBall = checkValue(attributes, "accurate_through_ball"),
                        TotalFinalThirdPasses = checkValue(attributes, "total_final_third_passes"),
                        SuccessfulFinalThirdPasses = checkValue(attributes, "successful_final_third_passes"),
                        TotalAttAssist = checkValue(attributes, "total_att_assist"),
                        OfftargetAttAssist = checkValue(attributes, "offtarget_att_assist"),
                        BigChanceCreated = checkValue(attributes, "big_chance_created"),
                        TotalOffside = checkValue(attributes, "total_offside"),
                        Dipossessed = checkValue(attributes, "dispossessed"),

                        TotalScoringAtt = checkValue(attributes, "total_scoring_att"),
                        BlockedScoringAtt = checkValue(attributes, "blocked_scoring_att"),
                        OntargetScoringAtt = checkValue(attributes, "ontarget_scoring_att"),
                        ShotOffTarget = checkValue(attributes, "shot_off_target"),
                        PostScoringAtt = checkValue(attributes, "post_scoring_att"),
                        BigChanceMissed = checkValue(attributes, "big_chance_missed"),

                        TotalTackle = checkValue(attributes, "total_tackle"),
                        LastManTackle = checkValue(attributes, "last_man_tackle"),
                        TotalClearance = checkValue(attributes, "total_clearance"),
                        EffectiveClearance = checkValue(attributes, "effective_clearance"),
                        SixYardBlock = checkValue(attributes, "six_yard_block"),
                        ClearanceOffLine = checkValue(attributes, "clearance_off_line"),
                        ErrorLeadToGoal = checkValue(attributes, "error_lead_to_goal"),
                        ErrorLeadToShot = checkValue(attributes, "error_lead_to_shot"),
                        Interception = checkValue(attributes, "interception"),
                        InterceptionWon = checkValue(attributes, "interception_won"),
                        BallRecovery = checkValue(attributes, "ball_recovery"),

                        CornerTaken = checkValue(attributes, "won_corners"),
                        TotalThrows = checkValue(attributes, "total_throws"),
                        PenaltyConceded = checkValue(attributes, "penalty_conceded"),
                        FkFoulWon = checkValue(attributes, "fk_foul_won"),
                        FkFoulLost = checkValue(attributes, "fk_foul_lost"),
                        YellowCard = checkValue(attributes, "yellow_card"),
                        RedCard = checkValue(attributes, "red_card"),
                        
                        AerialLost = checkValue(attributes, "aerial_lost"),
                        AerialWon = checkValue(attributes, "aerial_won"),
                        DuelLost = checkValue(attributes, "duel_lost"),
                        DuelWon = checkValue(attributes, "duel_won"),
                        WonContest = checkValue(attributes, "won_contest"),
                        TotalContest = checkValue(attributes, "total_contest"),
                        
                        PenaltySave = checkValue(attributes, "penalty_save"),
                    };
                    session.SaveOrUpdate(teamMatch);
                    tx.Commit();
                    currentTeamMatch = teamMatch;
                }
            }
        }

        /// <summary>
        /// Extracts player performance information and stores it in the database
        /// </summary>
        public void PlayerLine(string line)
        {
            string[] playerInformation = line.Split(',');
            var playerId = Convert.ToInt32(playerInformation[1]);
            var playerName = playerInformation[2];
            var playerRating = Convert.ToDouble(playerInformation[3], CultureInfo.InvariantCulture);
            var position = "";

            IDictionary<string, int> attributes = new Dictionary<string, int>();
            for (int i = 4; i < playerInformation.Length; i = i + 2)
            {
                if (playerInformation[i].Equals("position"))
                {
                    position = playerInformation[i + 1];
                }
                else if (playerInformation[i].StartsWith("rating"))
                {
                    //Skal vi ta vare på ratinginfo annet enn total rating?
                    continue;
                }
                else if (playerInformation[i].Length == 1)
                {
                    break;
                }
                else
                {
                    attributes.Add(playerInformation[i], Convert.ToInt32(playerInformation[i + 1]));
                }
            }

            //Update Player table in DB
            using (var session = NHibernateConfig.OpenSession())
            {
                using (var tx = session.BeginTransaction())
                {
                    Player player = session.Get<Player>(playerId);
                    if (player == null)
                    {
                        player = new Player
                        {
                            PlayerId = playerId,
                            //Name = playerName
                        };

                    }

                    player.Name = playerName;

                    session.SaveOrUpdate(player);
                    tx.Commit();
                }
            }


            //Update PlayerMatch table in DB
            if (position != "") //No need to store statistics for players that did not appear in the match?
            {
                using (var session = NHibernateConfig.OpenSession())
                {
                    using (var tx = session.BeginTransaction())
                    {
                        PlayerMatch playerMatch = new PlayerMatch()
                        {
                            TeamMatch = currentTeamMatch,
                            Player = session.Get<Player>(playerId),

                            GameStarted = checkValue(attributes, "game_started") == 1,
                            MinsPlayed = checkValue(attributes, "mins_played"),
                            ManOfTheMatch = checkValue(attributes, "man_of_the_match") == 1,
                            Rating = playerRating,
                            Position = position,
                            FormationPlace = checkValue(attributes, "formation_place"),

                            Saves = checkValue(attributes, "saves"),
                            GoodHighClaim = checkValue(attributes, "good_high_claim"),
                            PenaltySave = checkValue(attributes, "penalty_save"),

                            Goals = checkValue(attributes, "goals"),
                            OwnGoals = checkValue(attributes, "own_goals"),
                            OwnGoalAccrued = checkValue(attributes, "own_goal_accrued"),
                            AttPenGoal = checkValue(attributes, "att_pen_goal"),

                            Touches = checkValue(attributes, "touches"),
                            UnsuccessfulTouch = checkValue(attributes, "unsuccessful_touch"),
                            GoalAssist = checkValue(attributes, "goal_assist"),
                            TotalPass = checkValue(attributes, "total_pass"),
                            AccuratePass = checkValue(attributes, "accurate_pass"),
                            TotalThroughBall = checkValue(attributes, "total_through_ball"),
                            AccurateThroughBall = checkValue(attributes, "accurate_through_ball"),
                            TotalFinalThirdPasses = checkValue(attributes, "total_final_third_passes"),
                            SuccessfulFinalThirdPasses = checkValue(attributes, "successful_final_third_passes"),
                            TotalAttAssist = checkValue(attributes,"total_att_assist"),
                            OfftargetAttAssist = checkValue(attributes, "offtarget_att_assist"),
                            BigChanceCreated = checkValue(attributes, "big_chance_created"),
                            TotalOffside = checkValue(attributes, "total_offside"),
                            Dispossessed = checkValue(attributes, "dispossessed"),

                            TotalScoringAtt = checkValue(attributes, "total_scoring_att"),
                            BlockedScoringAtt = checkValue(attributes, "blocked_scoring_att"),
                            OntargetScoringAtt = checkValue(attributes, "ontarget_scoring_att"),
                            ShotOffTarget = checkValue(attributes, "shot_off_target"),
                            PostScoringAtt = checkValue(attributes, "post_scoring_att"),
                            BigChanceMissed = checkValue(attributes, "big_chance_missed"),
                            
                            TotalTackle = checkValue(attributes, "total_tackle"),
                            LastManTackle = checkValue(attributes, "last_man_tackle"),
                            TotalClearance = checkValue(attributes, "total_clearance"),
                            EffectiveClearance = checkValue(attributes, "effective_clearance"),
                            SixYardBlock = checkValue(attributes, "six_yard_block"),
                            ClearanceOffLine = checkValue(attributes, "clearance_off_line"),
                            ErrorLeadToGoal = checkValue(attributes, "error_lead_to_goal"),
                            ErrorLeadToShot = checkValue(attributes, "error_lead_to_shot"),
                            Interception = checkValue(attributes, "interception"),
                            InterceptionWon = checkValue(attributes, "interception_won"),
                            BallRecovery = checkValue(attributes, "ball_recovery"),

                            WonCorners = checkValue(attributes, "won_corners"),
                            TotalThrows = checkValue(attributes, "total_throws"),
                            PenaltyConceded = checkValue(attributes, "penalty_conceded"),
                            Fouls = checkValue(attributes, "fouls"),
                            YellowCard = checkValue(attributes, "yellow_card"),
                            RedCard = checkValue(attributes, "red_card"),

                            AerialWon = checkValue(attributes, "aerial_won"),
                            AerialLost = checkValue(attributes, "aerial_lost"),
                            DuelWon = checkValue(attributes, "duel_won"),
                            DuelLost = checkValue(attributes, "duel_lost"),
                            WonContest = checkValue(attributes, "won_contest"),
                            TotalContest = checkValue(attributes, "total_contest"),
                        };
                        session.SaveOrUpdate(playerMatch);
                        tx.Commit();
                    }
                }
            }
        }

        /// <summary>
        /// Extracts information regarding probable starting lineups and stores it in the database.
        /// </summary>
        public void ProbableLineUpLine(string line)
        {
            var teamId = HomeTeamId;
            if (line.StartsWith("away")) teamId = AwayTeamId;

            line = line.Substring(6);
            string[] games = line.Split(',');

            int[] playerIds = new int[11];
            for (int i = 0; i < 11; i++)
            {
                playerIds[i] = Convert.ToInt32(games[i]);
            }

            using (var session = NHibernateConfig.OpenSession())
            {
                foreach (var player in playerIds)
                {
                    using (var tx = session.BeginTransaction())
                    {
                        if (session.Get<Player>(player) == null)
                        {
                            var newPlayer = new Player
                            {
                                PlayerId = player
                            };
                            session.Save(newPlayer);
                            tx.Commit();
                        }
                    }
                }

                using (var tx = session.BeginTransaction())
                {
                    var probableLineUp = new Mappings.ProbableLineUp
                    {
                        Match = session.Get<Mappings.Match>(MatchId),
                        Team = session.Get<Team>(teamId),

                        Player1 = session.Get<Player>(playerIds[0]),
                        Player2 = session.Get<Player>(playerIds[1]),
                        Player3 = session.Get<Player>(playerIds[2]),
                        Player4 = session.Get<Player>(playerIds[3]),
                        Player5 = session.Get<Player>(playerIds[4]),
                        Player6 = session.Get<Player>(playerIds[5]),
                        Player7 = session.Get<Player>(playerIds[6]),
                        Player8 = session.Get<Player>(playerIds[7]),
                        Player9 = session.Get<Player>(playerIds[8]),
                        Player10 = session.Get<Player>(playerIds[9]),
                        Player11 = session.Get<Player>(playerIds[10]),
                    };
                    session.SaveOrUpdate(probableLineUp);
                    tx.Commit();
                }
            }
        }

        /// <summary>
        /// Checks if a Dictionary<string, int> contains a specified key.
        /// Returns the value for that key if it exists, or 0.
        /// </summary>
        private int checkValue(IDictionary<string, int> attributes, string key)
        {
            return attributes.ContainsKey(key) ? attributes[key] : 0;
        }
    }
}