﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

using FootballPredictions.Mappings;

using NHibernate.Linq;
using FootballPredictions.Evaluation.Betting.Plots;

namespace FootballPredictions.Evaluation.Betting
{
    /// <summary>
    /// Super class extended by all the betting strategies.
    /// </summary>
    public abstract class Betting
    {
        public List<Bet> Bets = new List<Bet>();
        public List<Prediction> Predictions = new List<Prediction>();
        public List<Odds> Odds = new List<Odds>();
        public int Stage;
        public string BettingStrategy;

        public Simulation Simulation { get; set; }
        public Bookmaker Bookmaker { get; set; }

        public double SafetyMargin { get; set; }

        /// <summary>
        /// Decide whether to bet on a match. Betting strategies should use this method for placing bets.
        /// </summary>
        public abstract void BetOnIndex(int i);

        /// <summary>
        /// Usually called from constructor in subclass for setting up and running betting simulation.
        /// </summary>
        public void Init(int simulationNo, string bookmakerId, double safetyMargin)
        {
            using (var session = NHibernateConfig.OpenSession())
            {
                Simulation = session.Get<Simulation>(simulationNo);
                Bookmaker = session.Get<Bookmaker>(bookmakerId);
            }
            SafetyMargin = safetyMargin;
            AssignVariables();
            PlaceBets();
            CreatePlots();
        }

        /// <summary>
        /// Place bets on matches where predictions are generated.
        /// </summary>
        public void PlaceBets()
        {
            if (Constants.SEASON_SIMULATION)
            {
                for (int i = Constants.SIZE_OF_TRAINING_SET; i < Predictions.Count; i++)
                {
                    BetOnIndex(i);
                }
            }
            else
            {
                for (int i = 0; i < Predictions.Count; i++)
                {
                    BetOnIndex(i);
                }
            }
        }

        /// <summary>
        /// Populate prediction list.
        /// </summary>
        public void AssignVariables()
        {
            using (var session = NHibernateConfig.OpenSession())
            {
                var predictions = from p in session.Query<Prediction>().ToList()
                                  where p.Simulation.SimulationId == Simulation.SimulationId
                                  select p;

                foreach (var p in predictions)
                {
                    Predictions.Add(p);
                }
                Stage = Predictions.First().Match.Stage.StageId;
            }
        }

        /// <summary>
        /// Calculate return of placed bets.
        /// </summary>
        public void CalculateReturn(char matchResult, Odds odds, Prediction prediction, double[] betsPlaced)
        {
            var ret = 0.0;

            if (matchResult == 'H')
            {
                ret += odds.HomeOdds * betsPlaced[0];
            }
            if (matchResult == 'D')
            {
                ret += odds.DrawOdds * betsPlaced[1];
            }
            if (matchResult == 'A')
            {
                ret += odds.AwayOdds * betsPlaced[2];
            }
            Bets.Add(new Bet()
            {
                BetsPlaced = betsPlaced,
                Match = odds.Match,
                Odds = odds,
                Prediction = prediction,
                Return = ret - (betsPlaced[0] + betsPlaced[1] + betsPlaced[2])
            });
        }

        /// <summary>
        /// Calculate expected return per unit
        /// </summary>
        public double ExpectedReturn(double odds, double probability)
        {
            return probability * odds - Constants.UNIT;
        }

        private void CreatePlots()
        {
            //AccumulatedIncome incomePlot = new AccumulatedIncome()
            //    {
            //        Bets = Bets,
            //        SimulationName = Simulation.Name,
            //        Stage = Stage,
            //        BettingStrategy = BettingStrategy
            //    };
            //incomePlot.Plot();
            //AccumulatedIncomePercentage incomePercent = new AccumulatedIncomePercentage()
            //    {
            //        Bets = Bets,
            //        SimulationName = Simulation.Name,
            //        Stage = Stage,
            //        BettingStrategy = BettingStrategy
            //    };
            //incomePercent.Plot();
            //BetsPlacedOverview bpo = new BetsPlacedOverview()
            //    {
            //        Bets = Bets,
            //        BettingStrategy = BettingStrategy,
            //        SimulationName = Simulation.Name,
            //        Stage = Stage
            //    };
            //bpo.Plot();
            //BetProbabilityPlot bpp = new BetProbabilityPlot()
            //    {
            //        Bets = Bets,
            //        BettingStrategy = BettingStrategy,
            //        SimulationName = Simulation.Name,
            //        Stage = Stage
            //    };
            //bpp.Plot();
            //TableOfBets tob = new TableOfBets()
            //    {
            //        Bets = Bets,
            //        BettingStrategy = BettingStrategy,
            //        SimulationName = Simulation.Name,
            //        Stage = Stage,
            //        StartDate = Constants.START_DATE_FOR_PLOTS
            //    };
            //tob.Plot();
            //CompareWithOdds cwo = new CompareWithOdds()
            //{
            //    Bets = Bets,
            //    BettingStrategy = BettingStrategy,
            //    SimulationName = Simulation.Name,
            //    Stage = Stage,
            //    StartDate = Constants.START_DATE_FOR_PLOTS
            //};
            ExpectedReturn er = new ExpectedReturn()
            {
                Bets = Bets,
                BettingStrategy = BettingStrategy,
                SimulationName = Simulation.Name,
                Stage = Stage
            };
            er.Plot();
            StatisticsTable st = new StatisticsTable()
            {
                Bets = Bets,
                BettingStrategy = BettingStrategy,
                SimulationName = Simulation.Name,
                Stage = Stage,
                StartDate = Constants.START_DATE_FOR_PLOTS
            };
            st.Plot();
        }
    }
}