﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

using FootballPredictions.Mappings;
using NHibernate.Linq;
using System.Globalization;

namespace FootballPredictions.Evaluation.Betting.Plots
{
    /// <summary>
    /// Scatter plot where the bets are placed according to the prediction and given odds.
    /// </summary>
    public class CompareWithOdds : BettingPlot
    {
        public DateTime StartDate { get; set; }

        public override void Plot()
        {
            var prefix = "\\begin{figure} \n"
                        + "\\centering \n"
                        + "\\begin{tikzpicture} \n"
                        + "\\begin{axis}\n"
                        + "[legend pos=outer north east, \n"
                        + "xlabel=Match, ylabel=Probability] \n";

            var suffix = "\\legend{1/Odds, Probability, Won, Lost, No bet} \n"
                        + "\\end{axis} \n"
                        + "\\end{tikzpicture}\\newline"
                        + "\\caption{<ENTER Figure Text>}% \n"
                        + "\\label{pgfplots:<ENTER Caption>}% \n"
                        + "\\end{figure}\n";

            var homePlot = prefix;
            homePlot += CreateLinePlot('H');
            homePlot += CreateScatterPlot('H');
            homePlot += suffix;

            var drawPlot = prefix;
            drawPlot += CreateLinePlot('D');
            drawPlot += CreateScatterPlot('D');
            drawPlot += suffix;

            var awayPlot = prefix;
            awayPlot += CreateLinePlot('A');
            awayPlot += CreateScatterPlot('A');
            awayPlot += suffix;

            using (System.IO.StreamWriter file = new System.IO.StreamWriter(Constants.TEXT_FILES + "/JAGS/BETTING/comparison_" + SimulationName + "_" + Stage + "_" + BettingStrategy + ".tex"))
            {
                file.Write(homePlot + drawPlot + awayPlot);
            }
        }

        private string CreateLinePlot(char version)
        {
            var plot = "";
            var colorCount = 0;
            var colors = new String[] { "blue", "orange" };

            plot += "\\addplot[color=" + colors[colorCount++ % 2] + "] coordinates {\n";
            for (int j = 0; j < Bets.Count(); j++)
            {
                if (Bets[j].Match.DateAndTime.CompareTo(StartDate) == -1)
                    continue;

                // Commented formulas are for comparing with normalized odds probabilities.
                switch (version)
                {
                    case ('H'):
                        {
                            plot += "(" + j + "," + (1.0 / Bets[j].Odds.HomeOdds).ToString(CultureInfo.GetCultureInfo("en-GB")) + ")\n";
                            //plot += "(" + j + "," + ((1 / Bets[j].Odds.HomeOdds) / 
                                //((1 / Bets[j].Odds.HomeOdds) + (1 / Bets[j].Odds.DrawOdds) + (1 / Bets[j].Odds.AwayOdds))).ToString(CultureInfo.GetCultureInfo("en-GB")) + ")\n";
                            break;
                        }
                    case ('D'):
                        {
                            plot += "(" + j + "," + (1.0 / Bets[j].Odds.DrawOdds).ToString(CultureInfo.GetCultureInfo("en-GB")) + ")\n";
                            //plot += "(" + j + "," + ((1 / Bets[j].Odds.DrawOdds) / 
                            //    ((1 / Bets[j].Odds.HomeOdds) + (1 / Bets[j].Odds.DrawOdds) + (1 / Bets[j].Odds.AwayOdds))).ToString(CultureInfo.GetCultureInfo("en-GB")) + ")\n";
                            break;
                        }
                    case ('A'):
                        {
                            plot += "(" + j + "," + (1.0 / Bets[j].Odds.AwayOdds).ToString(CultureInfo.GetCultureInfo("en-GB")) + ")\n";
                            //plot += "(" + j + "," + ((1 / Bets[j].Odds.AwayOdds) / 
                            //    ((1 / Bets[j].Odds.HomeOdds) + (1 / Bets[j].Odds.DrawOdds) + (1 / Bets[j].Odds.AwayOdds))).ToString(CultureInfo.GetCultureInfo("en-GB")) + ")\n";
                            break;
                        }
                    default: break;
                }
            }
            plot += "};\n";

            plot += "\\addplot[color=" + colors[colorCount++ % 2] + "] coordinates {\n";
            for (int j = 0; j < Bets.Count(); j++)
            {
                if (Bets[j].Match.DateAndTime.CompareTo(StartDate) == -1)
                    continue;

                switch (version)
                {
                    case ('H'):
                        {
                            plot += "(" + j + "," + Bets[j].Prediction.HomeProbability.ToString(CultureInfo.GetCultureInfo("en-GB")) + ")\n";
                            break;
                        }
                    case ('D'):
                        {
                            plot += "(" + j + "," + Bets[j].Prediction.DrawProbability.ToString(CultureInfo.GetCultureInfo("en-GB")) + ")\n";
                            break;
                        }
                    case ('A'):
                        {
                            plot += "(" + j + "," + Bets[j].Prediction.AwayProbability.ToString(CultureInfo.GetCultureInfo("en-GB")) + ")\n";
                            break;
                        }
                    default: break;
                }
            }
            plot += "};\n";

            return plot;
        }

        private string CreateScatterPlot(char version)
        {
            var plot = "\\addplot[scatter,only marks,\n"
                        + "point meta=explicit symbolic,\n"
                        + "mark size=0.6pt,\n"
                        + "scatter/classes=\n"
                        + "{a={mark=oplus*,green},b={mark=halfcircle*,red},c={mark=o,black}}]\n"
                        + "table[meta=label] {\n"
                        + "x y label\n";

            var coordinates = "";
            
            for (int i = 0; i < Bets.Count(); i ++)
            {
                if (Bets[i].Match.DateAndTime.CompareTo(StartDate) == -1)
                    continue;

                var x = 0.0;
                var y = 0.0;

                switch (version)
                {
                    case ('H'):
                        {
                            var label = "c";
                            if (Bets[i].BetsPlaced[0] > 0)
                                if (Bets[i].Return > 0)
                                    label = "a";
                                else
                                    label = "b";

                            coordinates += i + " " + Bets[i].Prediction.HomeProbability.ToString(CultureInfo.GetCultureInfo("en-GB")) + " " + label + "\n";
                            break;
                        }
                    case ('D'):
                        {
                            var label = "c";
                            if (Bets[i].BetsPlaced[1] > 0)
                                if (Bets[i].Return > 0)
                                    label = "a";
                                else
                                    label = "b";

                            coordinates += i + " " + Bets[i].Prediction.DrawProbability.ToString(CultureInfo.GetCultureInfo("en-GB")) + " " + label + "\n";
                            break;
                        }
                    case ('A'):
                        {
                            var label = "c";
                            if (Bets[i].BetsPlaced[2] > 0)
                                if (Bets[i].Return > 0)
                                    label = "a";
                                else
                                    label = "b";

                            coordinates += i + " " + Bets[i].Prediction.AwayProbability.ToString(CultureInfo.GetCultureInfo("en-GB")) + " " + label + "\n";
                            break;
                        }
                    default: break;
                }
            }
            plot += coordinates + "};\n";
            return plot;
        }
    }
}