﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

using NHibernate.Linq;

using FootballPredictions.Mappings;

namespace FootballPredictions.Evaluation
{
    /// <summary>
    /// A class to calculate the Rank Probability Score (RPS) as described by Anthony Constantinou 
    /// and Norman E. Fenton in the paper "Solving the problem of inadequate scoring rules for 
    /// assessing probabilistic football forecast models"
    /// </summary>
    public class RPS
    {
        public List<double> RPSs = new List<double>();
        public double SumRps { get; set; }
        public double AvgRps { get; set; }
        
        public RPS(int i)
        {
            using (var session = NHibernateConfig.OpenSession())
            {
                CalculateRPS(session.Get<Simulation>(i));
            }
        }

        public RPS(Simulation simulation)
        {
            CalculateRPS(simulation);
        }
        
        private List<double> CalculateRPS(Simulation simulation)
        {
            var r = 3;
            double[] p = new double[3];
            int[] e = new int[3];
            AvgRps = 0.0;
            SumRps = 0.0;

            var RPS = 0.0;
                       

            using (var session = NHibernateConfig.OpenSession())
            {
                var predictions = from pred in session.Query<Prediction>().ToList()
                                  where pred.Simulation.SimulationId == simulation.SimulationId
                                  select pred;

                predictions = predictions.OrderBy(pred => pred.Match.DateAndTime);

                foreach (var prediction in predictions)
                {
                    RPS = 0.0;
                    
                    switch(prediction.Match.Result)
                    {
                        case('H'):
                            {
                                e = new int[]{1,1,1};
                                break;
                            }
                        case('D'):
                            {
                                e = new int[]{0,1,1};
                                break;
                            }
                        case('A'):
                            {
                                e = new int[]{0,0,1};
                                break;
                            }
                    }
                   
                    //Probabilities predicted by prediction system.
                    p = new double[]{prediction.HomeProbability, 
                        prediction.DrawProbability + prediction.HomeProbability, 
                        prediction.AwayProbability + prediction.HomeProbability + prediction.DrawProbability};

                    // Probabilities reported by Clarke and Norman 1995). Use these for generating RPS plot used for comparisons.
                    //p = new double[]{0.487, 0.487 + 0.267, 0.487 + 0.267 + 0.246};

                    var tempi = 0.0;
                    for (int i = 0; i < r; i++)
                    {
                         tempi += Math.Pow((p[i] - e[i]), 2);
                    }
                    RPS = 1.0 / ((double)r - 1.0) * tempi;
                    SumRps += RPS;
                    RPSs.Add(RPS);
                }
            }

            if (Constants.SEASON_SIMULATION)
            {
                AvgRps = SumRps / ((double)(RPSs.Count - Constants.SIZE_OF_TRAINING_SET));
            }
            else
            {
                AvgRps = SumRps / (double)RPSs.Count;
            }
            return RPSs;
        }
    }
}