﻿using FootballPredictions.JAGS;
using SharpJags.Jags;
using SharpJags.Models;
using SharpJags.Parsing;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

using NHibernate.Linq;
using FootballPredictions.Mappings;
using SharpJags.Math;
using System.IO;
using System.Globalization;

namespace FootballPredictions.JAGS
{
    /// <summary>
    /// This class is for creating predictions from weighted votes from different prediction models. 
    /// It was created for testing if a combination of the different models would result in more accurate predictions. 
    /// However, it does not work as intended at this time.
    /// </summary>
    public class AggregatedSimulator
    {

        public string ModelDefinitionName { get; set; }
        public string ModelDefinitionDefinition { get; set; }
        public List<JagsMonitor> JagsMonitors = new List<JagsMonitor>();
        public int[] Simulations;
        public List<List<Prediction>> ModelPredictions = new List<List<Prediction>>(); 
        public Matrix<double> ModelProbs;
        public double[] Weights;
        public int SamplesAfterThinning = (int)Math.Ceiling((double)Constants.SAMPLE_COUNT / (double)Constants.THINNING);

        public AggregatedSimulator(int[] simulations)
        {
            Simulations = simulations;
            ModelDefinitionName = "AggregatedBet";
            ModelDefinitionDefinition =
@"#
        
# HL, 2012-07-19

#

data{

    for (i in 1:noGames){

        ones[i] <- 1

    }

}

model {

    weights ~ ddirch(weightPrior)

    for (i in 1:noGames){       

       ones[i] ~ dbern( inprod(weights,  t(modelProbs[i, ])) )

    }

}";
            JagsMonitors.Add(new JagsMonitor()
            {
                ParameterName = "weights",
                Thin = Constants.THINNING
            });
            Initialize();
            ParseCodaChain();
            CreateWeightedPredictions();
        }
              
        public void Initialize()
        {
            var modelDefinition = new ModelDefinition
            {
                Name = ModelDefinitionName,
                Definition = ModelDefinitionDefinition
            };

  
            JagsConfig.BinPath = Constants.BIN_PATH;
            var run = new JagsRun
            {
                WorkingDirectory = Constants.TEXT_FILES + "JAGS/",
                ModelDefinition = modelDefinition,

                ModelData = GetModelData(),
                Monitors = JagsMonitors,
                Parameters = new MCMCParameters
                {
                    BurnIn = Constants.BURN_IN,
                    SampleCount = Constants.SAMPLE_COUNT,
                    Chains = Constants.CHAINS
                },
            };
            ISampleParser parser = new ModifiedCodaParser();
            JagsWrapper.Run(run, parser);
        }

        private JagsData GetModelData()
        {
            var noGames = 0;
            using (var session = NHibernateConfig.OpenSession())
            {
                var pred = from p in session.Query<Prediction>().ToList()
                            where p.Simulation.SimulationId == Simulations[0]
                            select p;
                noGames = pred.Count();
                ModelPredictions.Add(pred.ToList());
                ModelProbs = new double[Simulations.Count(), noGames];
                var counter = 0;
                foreach (var p in pred)
                {
                    if (p.Match.Result == 'H')
                        ModelProbs[0, counter] = p.HomeProbability;
                    else if (p.Match.Result == 'D')
                        ModelProbs[0, counter] = p.DrawProbability;
                    else if (p.Match.Result == 'A')
                        ModelProbs[0, counter] = p.AwayProbability;
                    counter++;
                }
                for (int i=1; i<Simulations.Count(); i++)
                {
                    pred = from p in session.Query<Prediction>().ToList()
                               where p.Simulation.SimulationId == Simulations[i]
                               select p;
                    ModelPredictions.Add(pred.ToList());
                    if (pred.Count() != ModelProbs.Cols)
                    {
                        throw new Exception();
                    }
                    counter = 0;
                    foreach (var p in pred)
                    {
                        if (p.Match.Result == 'H')
                            ModelProbs[i, counter] = p.HomeProbability;
                        else if (p.Match.Result == 'D')
                            ModelProbs[i, counter] = p.DrawProbability;
                        else if (p.Match.Result == 'A')
                            ModelProbs[i, counter] = p.AwayProbability;
                        counter++;
                    }
                }
            }

            Matrix<double> weightPrior = new double[Simulations.Count(),1];
            for (int i = 0; i < Simulations.Count(); i++ )
            {
                weightPrior[i,0] = 1.0;
            }

            return new JagsData
                {
                    {"modelProbs" , ModelProbs },
                    {"weightPrior" , weightPrior },
                    {"noGames" , noGames },
                };
        }

        private void ParseCodaChain()
        {
            Weights = new double[Simulations.Count()];
            for (int i = 0; i < Constants.CHAINS; i++)
            {
                StreamReader fileIndex = new StreamReader(Constants.TEXT_FILES + "/JAGS/CODA" + i + "index.txt");
                StreamReader fileChain = new StreamReader(Constants.TEXT_FILES + "/JAGS/CODA" + i + "chain1.txt");
                var indexLine = "";
                while ((indexLine = fileIndex.ReadLine()) != null)
                {
                    for (int j = 1; j <= Constants.SIMULATION_IDS_AGGREGATE.Count(); j++)
                    {
                        if (indexLine.StartsWith("weights[" + j + "]"))
                        {
                            for (int k = 0; k < SamplesAfterThinning; k++)
                            {
                                var chainLine = fileChain.ReadLine();
                                Weights[j - 1] += Convert.ToDouble(chainLine.Split(' ')[2], CultureInfo.GetCultureInfo("en-GB"));
                            }
                        }
                    }
                }
                fileIndex.Close();
                fileChain.Close();
            }
            for (int i = 0; i < Constants.SIMULATION_IDS_AGGREGATE.Count(); i++)
            {
                Weights[i] = Weights[i] / (double)(Constants.CHAINS*SamplesAfterThinning);
            }
        }

        private void CreateWeightedPredictions()
        {
            using (var session = NHibernateConfig.OpenSession())
            {
                object simulation;
                using (var tx = session.BeginTransaction())
                {
                    var sims = "";
                    foreach (var s in Simulations)
                    {
                        sims = sims + "_" + s;
                    }
                    Simulation sim = new Simulation()
                    {
                        Name = this.GetType().Name + "_sim" + sims,
                    };
                    simulation = session.Save(sim);
                    tx.Commit();
                }
                // Loop over all predictions
                for (int i = 0; i < ModelPredictions[0].Count; i++)
                {
                    var homeProb = 0.0;
                    var drawProb = 0.0;
                    var awayProb = 0.0;
                    var homeStrength = new TeamStrength()
                        {
                            Attack = 0.0,
                            Defence = 0.0
                        };
                    var awayStrength = new TeamStrength()
                    {
                        Attack = 0.0,
                        Defence = 0.0
                    };
                    var homeGoals = 0.0;
                    var awayGoals = 0.0;

                    // Loop over all models
                    for (int j = 0; j < ModelPredictions.Count(); j++)
                    {
                        homeProb += ModelPredictions[j][i].HomeProbability * Weights[j];
                        drawProb += ModelPredictions[j][i].DrawProbability * Weights[j];
                        awayProb += ModelPredictions[j][i].AwayProbability * Weights[j];

                        homeStrength.Attack += session.Get<TeamStrength>(ModelPredictions[j][i].HomeStrength.TeamStrengthId).Attack * Weights[j];
                        homeStrength.Defence += session.Get<TeamStrength>(ModelPredictions[j][i].HomeStrength.TeamStrengthId).Defence * Weights[j];
                        awayStrength.Attack += session.Get<TeamStrength>(ModelPredictions[j][i].AwayStrength.TeamStrengthId).Attack * Weights[j];
                        awayStrength.Defence += session.Get<TeamStrength>(ModelPredictions[j][i].AwayStrength.TeamStrengthId).Defence * Weights[j];

                        homeGoals += (double)session.Get<Result>(ModelPredictions[j][i].Result.ResultId).HomeGoals * Weights[j];
                        awayGoals += (double)session.Get<Result>(ModelPredictions[j][i].Result.ResultId).AwayGoals * Weights[j];
                    }
                    var match = ModelPredictions[0][i].Match;

                    object resultId;
                    using (var tx = session.BeginTransaction())
                    {
                        var result = from r in session.Query<Result>().ToList()
                                     where r.HomeGoals == (int)Math.Round(homeGoals)
                                     && r.AwayGoals == (int)Math.Round(awayGoals)
                                     select r;
                        if (result.Count<Result>() == 0)
                        {
                            var newResult = new Result()
                            {
                                HomeGoals = (int)Math.Round(homeGoals),
                                AwayGoals = (int)Math.Round(awayGoals)
                            };
                            resultId = session.Save(newResult);
                            tx.Commit();
                        }
                        else
                        {
                            resultId = result.First<Result>().ResultId;
                        }
                    }
                    object homeStrengthId;
                    object awayStrengthId;
                    using (var tx = session.BeginTransaction())
                    {
                        var homeTeamStrength = new TeamStrength()
                        {
                            Attack = homeStrength.Attack,
                            Defence = homeStrength.Defence
                        };
                        var awayTeamStrength = new TeamStrength()
                        {
                            Attack = awayStrength.Attack,
                            Defence = awayStrength.Defence
                        };
                        homeStrengthId = session.Save(homeTeamStrength);
                        awayStrengthId = session.Save(awayTeamStrength);
                        tx.Commit();
                    }

                    using (var tx = session.BeginTransaction())
                    {
                        Prediction newPred = new Prediction()
                        {
                            HomeProbability = homeProb,
                            DrawProbability = drawProb,
                            AwayProbability = awayProb,
                            Match = match,
                            HomeStrength = session.Get<TeamStrength>(homeStrengthId),
                            AwayStrength = session.Get<TeamStrength>(awayStrengthId),
                            Result = session.Get<Result>(resultId),
                            Simulation = session.Get<Simulation>(simulation),
                        };
                        session.Save(newPred);
                        tx.Commit();
                    }
                }
            }

        }
    }
}