﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

using NHibernate.Linq;

using FootballPredictions.Mappings;

namespace FootballPredictions.JAGS
{
    /// <summary>
    /// Since we found that the SharpJags package did not create new CodaFiles for each chain during simulations, this class is created for combining several runs with the same parameters.
    /// </summary>
    public class CombineSimulations
    {
        public int[] SimulationsToCombine;

        public CombineSimulations(int[] simulations)
        {
            SimulationsToCombine = simulations;
        }

        public void Combine()
        {
            List<List<Prediction>> predictionsList = new List<List<Prediction>>();
            Simulation simulation;

            using (var session = NHibernateConfig.OpenSession())
            {
                simulation = session.Get<Simulation>(SimulationsToCombine[0]);
                var burnIn = simulation.BurnIn;
                var sampleCount = simulation.SampleCount;
                var chains = simulation.Chains;
                var name = simulation.Name;

                for (int i = 0; i < SimulationsToCombine.Length; i++)
                {
                    if (i != 0)
                    {
                        simulation = session.Get<Simulation>(SimulationsToCombine[i]);
                        chains += simulation.Chains;

                        if (sampleCount != simulation.SampleCount || burnIn != simulation.BurnIn || !name.Equals(simulation.Name))
                        {
                            throw new Exception("Simulations have to have the same value for name, number of samples and burn in in order to be combined!");
                        }
                    }
                    var predictions = from p in session.Query<Prediction>().ToList()
                                      where p.Simulation.SimulationId == SimulationsToCombine[i]
                                      select p;
                    predictionsList.Add(predictions.ToList());
                }
                //Lag ny simulering
                Simulation newSim;
                using (var tx = session.BeginTransaction())
                {
                    newSim = new Simulation()
                    {
                        BurnIn = burnIn,
                        Chains = chains,
                        Name = name,
                        SampleCount = sampleCount,
                        StartTime = DateTime.Now
                    };
                    session.Save(newSim);
                    tx.Commit();
                }

                List<Prediction> newPredictions = new List<Prediction>();
                
                for (int i = 0; i < predictionsList[0].Count(); i++)
                {
                    var match = session.Get<Match>(predictionsList[0][i].Match.MatchId);
                    
                    var homeProbability = 0.0;
                    var drawProbability = 0.0;
                    var awayProbability = 0.0;
                    var homeStrength = new TeamStrength();
                    var awayStrength = new TeamStrength();
                    var result = new Result();

                    for (int j = 0; j < SimulationsToCombine.Length; j++)
                    {
                        homeProbability += predictionsList[j][i].HomeProbability;
                        drawProbability += predictionsList[j][i].DrawProbability;
                        awayProbability += predictionsList[j][i].AwayProbability;
                        homeStrength.Attack += session.Get<TeamStrength>(predictionsList[j][i].HomeStrength.TeamStrengthId).Attack;
                        homeStrength.Defence += session.Get<TeamStrength>(predictionsList[j][i].HomeStrength.TeamStrengthId).Defence;
                        awayStrength.Attack += session.Get<TeamStrength>(predictionsList[j][i].AwayStrength.TeamStrengthId).Attack;
                        awayStrength.Defence += session.Get<TeamStrength>(predictionsList[j][i].AwayStrength.TeamStrengthId).Defence;
                        result.HomeGoals += session.Get<Result>(predictionsList[j][i].Result.ResultId).HomeGoals;
                        result.AwayGoals += session.Get<Result>(predictionsList[j][i].Result.ResultId).AwayGoals;
                    }
                    homeProbability = homeProbability / (double)SimulationsToCombine.Length;
                    drawProbability = drawProbability / (double)SimulationsToCombine.Length;
                    awayProbability = awayProbability / (double)SimulationsToCombine.Length;
                    homeStrength.Attack = homeStrength.Attack / (double)SimulationsToCombine.Length;
                    homeStrength.Defence = homeStrength.Defence / (double)SimulationsToCombine.Length;
                    awayStrength.Attack = awayStrength.Attack / (double)SimulationsToCombine.Length;
                    awayStrength.Defence = awayStrength.Defence / (double)SimulationsToCombine.Length;
                    result.HomeGoals = DivideAndRound(result.HomeGoals, SimulationsToCombine.Length);
                    result.AwayGoals = DivideAndRound(result.AwayGoals, SimulationsToCombine.Length);

                    using (var tx = session.BeginTransaction())
                    {
                        session.Save(homeStrength);
                        session.Save(awayStrength);
                        session.SaveOrUpdate(result);
                        tx.Commit();
                    }

                    newPredictions.Add(new Prediction()
                    {
                        Simulation = newSim,
                        HomeProbability = homeProbability,
                        DrawProbability = drawProbability,
                        AwayProbability = awayProbability,
                        HomeStrength = homeStrength,
                        AwayStrength = awayStrength,
                        Match = match,
                        Result = result
                    });
                }
                using (var tx = session.BeginTransaction())
                {
                    foreach (var p in newPredictions)
                    {
                        session.Save(p);
                    }
                    tx.Commit();
                }
            }
        }

        private int DivideAndRound(int a, int b)
        {
            double remainder = Math.IEEERemainder((double)a, (double)b);
            return remainder >= 0 ? (int)Math.Floor((double)a/(double)b) : (int)Math.Ceiling((double)a/(double)b);
        }
    }
}