﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

using NHibernate.Linq;
using FootballPredictions.Mappings;
using SharpJags.Jags;
using SharpJags.Math;

namespace FootballPredictions.JAGS.DixonColesDynamic
{
    /// <summary>
    /// The set-up of the Dixon Coles predictive model.
    /// </summary>
    public class DixonColesDynamicPredictiveModel : PredictiveModel
    {

        public Matrix<double> ScheduleMX { get; set; }
        public Matrix<double> SortedGameMx { get; set; }

        public DixonColesDynamicPredictiveModel()
        {
            AssignVariables();
        }

        public override void AssignVariables()
        {
            using (var session = NHibernateConfig.OpenSession())
            {
                using (var tx = session.BeginTransaction())
                {
                    var matches = from g in session.Query<Match>().ToList()
                                  where g.Stage.StageId == Constants.STAGE_ID
                                  select g;

                    foreach (var match in matches)
                    {
                        if (!Teams.Contains(match.HomeTeam))
                        {
                            Teams.Add(match.HomeTeam);
                        }
                        if (!Teams.Contains(match.AwayTeam))
                        {
                            Teams.Add(match.AwayTeam);
                        }
                    }

                    matches = matches.OrderBy(m => m.DateAndTime);

                    foreach (var match in matches)
                    {
                        Matches.Add(match);
                    }

                    NoTeams = Teams.Count();
                    NoMatches = matches.Count();
                    NoRounds = (int)(((double)NoMatches / NoTeams) * 2);
                }
            }
            // Initialize the matrices
            TeamMx = new double[NoMatches, 2];
            GoalMx = new double[NoMatches, 2];
            RoundMx = new double[NoMatches, 2];
           
            ScheduleMX = new double[NoMatches, 4]; //round, homeTeam, round, awayTeam
            SortedGameMx = new double[NoTeams, NoRounds];

            var teamMatchesPlayed = new double[NoTeams];
            var matchidx = 0;

            // Fill the matrices here
            foreach (var match in Matches)
            {
                var homeTeamIdx = Teams.IndexOf(match.HomeTeam);
                var awayTeamIdx = Teams.IndexOf(match.AwayTeam);

                teamMatchesPlayed[homeTeamIdx]++;
                teamMatchesPlayed[awayTeamIdx]++;

                TeamMx[matchidx, 0] = homeTeamIdx + 1;
                TeamMx[matchidx, 1] = awayTeamIdx + 1;

                RoundMx[matchidx, 0] = teamMatchesPlayed[homeTeamIdx];
                RoundMx[matchidx, 1] = teamMatchesPlayed[awayTeamIdx];

                ScheduleMX[matchidx, 0] = RoundMx[matchidx, 0];
                ScheduleMX[matchidx, 1] = TeamMx[matchidx, 0];
                ScheduleMX[matchidx, 2] = RoundMx[matchidx, 1];
                ScheduleMX[matchidx, 3] = TeamMx[matchidx, 1];
                
                SortedGameMx[homeTeamIdx, (int)RoundMx[matchidx, 0] - 1] = teamMatchesPlayed[homeTeamIdx];
                SortedGameMx[awayTeamIdx, (int)RoundMx[matchidx, 1] - 1] = teamMatchesPlayed[awayTeamIdx];
                
                GoalMx[matchidx, 0] = match.HomeGoals;
                GoalMx[matchidx, 1] = match.AwayGoals;
                
                matchidx++;
            }
        }

        public override JagsData GetMatchSpecificJagsData(int matchNumber)
        {
            Matrix<double> goalMx = new double[matchNumber, 2];

            //var noRounds = (int)(((double)matchNumber / NoTeams) * 2);
            var noRounds = 38;

            Matrix<double> teamMx = new double[matchNumber, 2];
            Matrix<double> roundMx = new double[matchNumber, 2];

            Matrix<double> scheduleMX = new double[matchNumber, 4]; //round, homeTeam, round, awayTeam
            Matrix<double> sortedGameMx = new double[NoTeams, noRounds];

            for (int i = 0; i < matchNumber; i++)
            {
                goalMx[i, 0] = GoalMx[i, 0];
                goalMx[i, 1] = GoalMx[i, 1];

                teamMx[i, 0] = TeamMx[i, 0];
                teamMx[i, 1] = TeamMx[i, 1];

                roundMx[i, 0] = RoundMx[i, 0];
                roundMx[i, 1] = RoundMx[i, 1];

                scheduleMX[i, 0] = RoundMx[i, 0];
                scheduleMX[i, 1] = TeamMx[i, 0];
                scheduleMX[i, 2] = RoundMx[i, 1];
                scheduleMX[i, 3] = TeamMx[i, 1];
            }
            goalMx[matchNumber - 1, 0] = Double.NaN;
            goalMx[matchNumber - 1, 1] = Double.NaN;

            for (int j = 0; j < noRounds; j++)
            {
                for (int k = 0; k < NoTeams; k++)
                {
                    sortedGameMx[k, j] = SortedGameMx[k, j];
                }
            }

            return new JagsData
            {
                { "sortedGame", sortedGameMx },
                { "schedule", scheduleMX },
                { "goalsScored", goalMx },
                { "noGames", matchNumber },
                { "noRounds", noRounds},
                { "noTeams", NoTeams },
            };
        }

        public override JagsData GetSeasonSpecificData()
        {
            // Returns the matrices
            for (int i = Constants.SIZE_OF_TRAINING_SET; i < NoMatches; i++)
            {
                GoalMx[i, 0] = Double.NaN;
                GoalMx[i, 1] = Double.NaN;
            }
            
            return new JagsData
            {
                { "sortedGame", SortedGameMx },
                { "schedule", ScheduleMX },
                { "goalsScored", GoalMx },
                { "noGames", NoMatches },
                { "noRounds", NoRounds},
                { "noTeams", NoTeams },
            };
        }
        public override JagsData GetModelPriors()
        {
            return null;
        }
    }
}