﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

using NHibernate.Linq;
using FootballPredictions.Mappings;
using SharpJags.Jags;
using SharpJags.Math;


namespace FootballPredictions.JAGS.DixonColesDynamic
{
    /// <summary>
    /// Dixon Coles with information of goals scored by the starting line-up.
    /// </summary>
    public class IndividualDixonColesDynamicPredictiveModel : PredictiveModel
    {

        
        public List<PlayerMatch> PlayerMatches = new List<PlayerMatch>();
        public IDictionary<Player, Int32> PlayerGoalDictionary = new Dictionary<Player, Int32>();
        public IDictionary<Player, Int32> PlayerConcededDictionary = new Dictionary<Player, Int32>();
        public IDictionary<Player, Int32> PlayerMinutesPlayedDictionary = new Dictionary<Player, Int32>();
        public IDictionary<Match, ProbableLineUp> ProbableHomeLineups = new Dictionary<Match, ProbableLineUp>();
        public IDictionary<Match, ProbableLineUp> ProbableAwayLineups = new Dictionary<Match, ProbableLineUp>();
        public Matrix<double> AttackScaleMx { get; set; }
        public Matrix<double> ScheduleMx { get; set; }
        public Matrix<double> SortedGameMx { get; set; }

        public IndividualDixonColesDynamicPredictiveModel()
        {
            AssignVariables();
        }

        public override void AssignVariables()
        {
            GetSeasonInformationFromDB();
            InitializeMatrices();

        }

        /// <summary>
        /// Reads all season specific data needed for this simulation from database
        /// </summary>
        private void GetSeasonInformationFromDB()
        {
            using (var session = NHibernateConfig.OpenSession())
            {
                // Get all matches and teams
                var matches = from g in session.Query<Match>().ToList()
                                where g.Stage.StageId == Constants.STAGE_ID
                                select g;

                foreach (var match in matches)
                {
                    if (!Teams.Contains(match.HomeTeam))
                    {
                        Teams.Add(match.HomeTeam);
                    }
                    if (!Teams.Contains(match.AwayTeam))
                    {
                        Teams.Add(match.AwayTeam);
                    }
                }

                matches = matches.OrderBy(m => m.DateAndTime);

                foreach (var match in matches)
                {
                    Matches.Add(match);
                }

                NoTeams = Teams.Count();
                NoMatches = matches.Count();
                NoRounds = (int)(((double)NoMatches / NoTeams) * 2);

                //Get Players
                var playerMatches = from pm in session.Query<PlayerMatch>().ToList()
                                    where pm.TeamMatch.Match.Stage.StageId == Constants.STAGE_ID
                                    select pm;
                var players = from p in session.Query<Player>().ToList()
                              from pm in playerMatches
                              where p.PlayerId == pm.Player.PlayerId
                              select p;
                foreach (var playerMatch in playerMatches.Distinct())
                {
                    PlayerMatches.Add(playerMatch);
                }
                foreach (var player in players.Distinct())
                {
                    PlayerGoalDictionary.Add(player, 0);
                    PlayerMinutesPlayedDictionary.Add(player, 0);
                    PlayerConcededDictionary.Add(player, 0);
                }

                //Get probable lineups
                var probableLineups = from pl in session.Query<ProbableLineUp>().ToList()
                                      where pl.Match.Stage.StageId == Constants.STAGE_ID
                                      select pl;

                foreach (var probableLineup in probableLineups)
                {
                    if (probableLineup.Team.TeamId == probableLineup.Match.HomeTeam.TeamId)
                    {
                        ProbableHomeLineups.Add(probableLineup.Match, probableLineup);
                    }
                    else if (probableLineup.Team.TeamId == probableLineup.Match.AwayTeam.TeamId)
                    {
                        ProbableAwayLineups.Add(probableLineup.Match, probableLineup);
                    }
                }
            }
        }

        /// <summary>
        /// Initialize  and insert values in the matrices that are to be fed into JAGS.
        /// </summary>
        private void InitializeMatrices()
        {
            // Initialize the matrices
            TeamMx = new double[NoMatches, 2];
            GoalMx = new double[NoMatches, 2];
            RoundMx = new double[NoMatches, 2];
            AttackScaleMx = new double[NoTeams, NoRounds];
            ScheduleMx = new double[NoMatches, 4]; //roundHomeTeam, homeTeam, roundAwayTeam, awayTeam
            SortedGameMx = new double[NoTeams, NoRounds];
            
            int[] teamGoalsSoFar = new int[NoTeams];
            int[] goalsConcededSoFar = new int[NoTeams];
            List<PlayerMatch> homePlayersInvolved, awayPlayersInvolved;

            var teamMatchesPlayed = new double[NoTeams];

            var matchidx = 0;

            // Fill the matrices here
            foreach (var match in Matches)
            {
                var homeTeamIdx = Teams.IndexOf(match.HomeTeam);
                var awayTeamIdx = Teams.IndexOf(match.AwayTeam);

                teamMatchesPlayed[homeTeamIdx]++;
                teamMatchesPlayed[awayTeamIdx]++;

                TeamMx[matchidx, 0] = homeTeamIdx + 1;
                TeamMx[matchidx, 1] = awayTeamIdx + 1;

                RoundMx[matchidx, 0] = teamMatchesPlayed[homeTeamIdx];
                RoundMx[matchidx, 1] = teamMatchesPlayed[awayTeamIdx];

                ScheduleMx[matchidx, 0] = RoundMx[matchidx, 0];
                ScheduleMx[matchidx, 1] = TeamMx[matchidx, 0];
                ScheduleMx[matchidx, 2] = RoundMx[matchidx, 1];
                ScheduleMx[matchidx, 3] = TeamMx[matchidx, 1];

                SortedGameMx[homeTeamIdx, (int)RoundMx[matchidx, 0] - 1] = teamMatchesPlayed[homeTeamIdx];
                SortedGameMx[awayTeamIdx, (int)RoundMx[matchidx, 1] - 1] = teamMatchesPlayed[awayTeamIdx];
                                
                GoalMx[matchidx, 0] = match.HomeGoals;
                GoalMx[matchidx, 1] = match.AwayGoals;
                
                homePlayersInvolved = new List<PlayerMatch>();
                awayPlayersInvolved = new List<PlayerMatch>();

                //Get lineup contributions
                foreach (var playerMatch in PlayerMatches)
                {
                    if (playerMatch.TeamMatch.Match.MatchId == match.MatchId)
                    {
                        if (playerMatch.TeamMatch.Team.TeamId == match.HomeTeam.TeamId)
                        {
                            homePlayersInvolved.Add(playerMatch);
                        }
                        else
                        {
                            awayPlayersInvolved.Add(playerMatch);
                        }
                    }
                }
                homePlayersInvolved.OrderByDescending(hp => hp.MinsPlayed);
                awayPlayersInvolved.OrderByDescending(ap => ap.MinsPlayed);

                var homeProbableLineupAttackContribution = 0.0;
                var awayProbableLineupAttackContribution = 0.0;

                if (ProbableHomeLineups.ContainsKey(match))
                {
                    homeProbableLineupAttackContribution = CalculateLineupAttackContribution(ProbableHomeLineups[match], matchidx);
                }
                else
                {
                    homeProbableLineupAttackContribution = CalculateLineupAttackContribution(new ProbableLineUp
                    {
                        Player1 = homePlayersInvolved[0].Player,
                        Player2 = homePlayersInvolved[1].Player,
                        Player3 = homePlayersInvolved[2].Player,
                        Player4 = homePlayersInvolved[3].Player,
                        Player5 = homePlayersInvolved[4].Player,
                        Player6 = homePlayersInvolved[5].Player,
                        Player7 = homePlayersInvolved[6].Player,
                        Player8 = homePlayersInvolved[7].Player,
                        Player9 = homePlayersInvolved[8].Player,
                        Player10 = homePlayersInvolved[9].Player,
                        Player11 = homePlayersInvolved[10].Player,
                        Team = match.HomeTeam,
                        Match = match
                    }, matchidx);
                }
                if (ProbableAwayLineups.ContainsKey(match))
                {
                    awayProbableLineupAttackContribution = CalculateLineupAttackContribution(ProbableAwayLineups[match], matchidx);
                }
                else
                {
                    awayProbableLineupAttackContribution = CalculateLineupAttackContribution(new ProbableLineUp
                    {
                        Player1 = awayPlayersInvolved[0].Player,
                        Player2 = awayPlayersInvolved[1].Player,
                        Player3 = awayPlayersInvolved[2].Player,
                        Player4 = awayPlayersInvolved[3].Player,
                        Player5 = awayPlayersInvolved[4].Player,
                        Player6 = awayPlayersInvolved[5].Player,
                        Player7 = awayPlayersInvolved[6].Player,
                        Player8 = awayPlayersInvolved[7].Player,
                        Player9 = awayPlayersInvolved[8].Player,
                        Player10 = awayPlayersInvolved[9].Player,
                        Player11 = awayPlayersInvolved[10].Player,
                        Team = match.AwayTeam,
                        Match = match
                    }, matchidx);
                }

                var homeTeamTotalAttackContribution = (double)teamGoalsSoFar[homeTeamIdx] / Math.Max(1, (teamMatchesPlayed[homeTeamIdx] -1)*90);
                var awayTeamTotalAttackContribution = (double)teamGoalsSoFar[awayTeamIdx] / Math.Max(1, (teamMatchesPlayed[awayTeamIdx] -1)*90);

                double probableHomeLineupEfficiency;
                double probableAwayLineupEfficiency;

                probableHomeLineupEfficiency = homeTeamTotalAttackContribution == 0 ? 1 : homeProbableLineupAttackContribution / homeTeamTotalAttackContribution;
                probableAwayLineupEfficiency = awayTeamTotalAttackContribution == 0 ? 1 : awayProbableLineupAttackContribution / awayTeamTotalAttackContribution;

                AttackScaleMx[homeTeamIdx, (int)teamMatchesPlayed[homeTeamIdx] -1] = probableHomeLineupEfficiency == 0 ? 1 : probableHomeLineupEfficiency;
                AttackScaleMx[awayTeamIdx, (int)teamMatchesPlayed[awayTeamIdx] -1] = probableAwayLineupEfficiency == 0 ? 1 : probableAwayLineupEfficiency;

                //Update info before next round
                foreach (var playerMatch in homePlayersInvolved)
                {
                    if (playerMatch.MinsPlayed > 0)
                    {
                        PlayerGoalDictionary[playerMatch.Player] += playerMatch.Goals;
                        PlayerMinutesPlayedDictionary[playerMatch.Player] += playerMatch.MinsPlayed;
                    }
                }
                foreach (var playerMatch in awayPlayersInvolved)
                {
                    if (playerMatch.MinsPlayed > 0)
                    {
                        PlayerGoalDictionary[playerMatch.Player] += playerMatch.Goals;
                        PlayerMinutesPlayedDictionary[playerMatch.Player] += playerMatch.MinsPlayed;
                    }
                }
                teamGoalsSoFar[homeTeamIdx] += match.HomeGoals;
                teamGoalsSoFar[awayTeamIdx] += match.AwayGoals;

                goalsConcededSoFar[awayTeamIdx] += match.HomeGoals;
                goalsConcededSoFar[homeTeamIdx] += match.AwayGoals;

                matchidx++;
            }
        }

        /// <summary>
        /// Calculates the attack contribution of a starting lineup, based on scoring efficiency of all starting players.
        /// </summary>
        public double CalculateLineupAttackContribution(ProbableLineUp lineup, int matchidx)
        {
            var lineupContribution = 0.0;
            var avgMinsPlayed = 0.0;
            var goalsScored = 0.0;

            avgMinsPlayed += (PlayerMinutesPlayedDictionary[ControlPlayerKey(lineup.Player1)] +
                PlayerMinutesPlayedDictionary[ControlPlayerKey(lineup.Player2)] +
                PlayerMinutesPlayedDictionary[ControlPlayerKey(lineup.Player3)] +
                PlayerMinutesPlayedDictionary[ControlPlayerKey(lineup.Player4)] +
                PlayerMinutesPlayedDictionary[ControlPlayerKey(lineup.Player5)] +
                PlayerMinutesPlayedDictionary[ControlPlayerKey(lineup.Player6)] +
                PlayerMinutesPlayedDictionary[ControlPlayerKey(lineup.Player7)] +
                PlayerMinutesPlayedDictionary[ControlPlayerKey(lineup.Player8)] +
                PlayerMinutesPlayedDictionary[ControlPlayerKey(lineup.Player9)] +
                PlayerMinutesPlayedDictionary[ControlPlayerKey(lineup.Player10)] +
                PlayerMinutesPlayedDictionary[ControlPlayerKey(lineup.Player11)])
                / 11.0;
            goalsScored += (PlayerGoalDictionary[lineup.Player1] +
                            PlayerGoalDictionary[lineup.Player2] +
                            PlayerGoalDictionary[lineup.Player3] +
                            PlayerGoalDictionary[lineup.Player4] +
                            PlayerGoalDictionary[lineup.Player5] +
                            PlayerGoalDictionary[lineup.Player6] +
                            PlayerGoalDictionary[lineup.Player7] +
                            PlayerGoalDictionary[lineup.Player8] +
                            PlayerGoalDictionary[lineup.Player9] +
                            PlayerGoalDictionary[lineup.Player10] +
                            PlayerGoalDictionary[lineup.Player11]);
            lineupContribution = goalsScored / Math.Max(1, avgMinsPlayed);
            return lineupContribution;
        }

        /// <summary>
        /// Calculates the defence contribution of a starting lineup, based on how many goals the team concedes on average while these players are on the pitch.
        /// </summary>
        //public double CalculateLineupDefenceContribution(ProbableLineUp lineup, int matchidx)
        //{
        //    var lineupContribution = 0.0;
        //    lineupContribution += CalculatePlayerDefenceContribution(lineup.Player1);
        //    lineupContribution += CalculatePlayerDefenceContribution(lineup.Player2);
        //    lineupContribution += CalculatePlayerDefenceContribution(lineup.Player3);
        //    lineupContribution += CalculatePlayerDefenceContribution(lineup.Player4);
        //    lineupContribution += CalculatePlayerDefenceContribution(lineup.Player5);
        //    lineupContribution += CalculatePlayerDefenceContribution(lineup.Player6);
        //    lineupContribution += CalculatePlayerDefenceContribution(lineup.Player7);
        //    lineupContribution += CalculatePlayerDefenceContribution(lineup.Player8);
        //    lineupContribution += CalculatePlayerDefenceContribution(lineup.Player9);
        //    lineupContribution += CalculatePlayerDefenceContribution(lineup.Player10);
        //    lineupContribution += CalculatePlayerDefenceContribution(lineup.Player11);

        //    return lineupContribution / 11.0;
        //}

        /// <summary>
        /// Returns goal efficiency of a player
        /// </summary>
        private double CalculatePlayerAttackContribution(Player player)
        {
            var playerContribution = 0.0;
            if (PlayerGoalDictionary.ContainsKey(player))
                playerContribution = (double)PlayerGoalDictionary[player] / (double)Math.Max(1, PlayerMinutesPlayedDictionary[player]);
            else
            {
                PlayerGoalDictionary.Add(player, 0);
                PlayerConcededDictionary.Add(player, 0);
                PlayerMinutesPlayedDictionary.Add(player, 0);
            }
            return playerContribution;
        }

        private Player ControlPlayerKey(Player player)
        {
            if (!PlayerGoalDictionary.ContainsKey(player))
            {
                PlayerGoalDictionary.Add(player, 0);
                PlayerConcededDictionary.Add(player, 0);
                PlayerMinutesPlayedDictionary.Add(player, 0);
            }
            return player;
        }

        /// <summary>
        /// Returns total goals conceded in matches where this player appeared on the pitch.
        /// </summary>
        //private double CalculatePlayerDefenceContribution(Player player)
        //{
        //    var playerContribution = 0.0;
        //    if (PlayerConcededDictionary.ContainsKey(player))
        //        playerContribution = (double)PlayerConcededDictionary[player];
        //    else
        //    {
        //        PlayerConcededDictionary.Add(player, 0);
        //        PlayerGoalDictionary.Add(player, 0);
        //        PlayerMatchesPlayedDictionary.Add(player, 0);
        //    }
        //    return playerContribution;
        //}


        public override JagsData GetMatchSpecificJagsData(int matchNumber)
        {
            Matrix<double> goalMx = new double[matchNumber, 2];

            //var noRounds = (int)Math.Ceiling((((double)matchNumber / (double)NoTeams) * 2));
            var noRounds = 38;

            Matrix<double> teamMx = new double[matchNumber, 2];
            Matrix<double> roundMx = new double[matchNumber, 2];

            Matrix<double> scheduleMX = new double[matchNumber, 4]; //round, homeTeam, round, awayTeam
            Matrix<double> sortedGameMx = new double[NoTeams, noRounds];

            for (int i = 0; i < matchNumber; i++)
            {
                goalMx[i, 0] = GoalMx[i, 0];
                goalMx[i, 1] = GoalMx[i, 1];

                teamMx[i, 0] = TeamMx[i, 0];
                teamMx[i, 1] = TeamMx[i, 1];

                roundMx[i, 0] = RoundMx[i, 0];
                roundMx[i, 1] = RoundMx[i, 1];

                scheduleMX[i, 0] = RoundMx[i, 0];
                scheduleMX[i, 1] = TeamMx[i, 0];
                scheduleMX[i, 2] = RoundMx[i, 1];
                scheduleMX[i, 3] = TeamMx[i, 1];
            }
            goalMx[matchNumber - 1, 0] = Double.NaN;
            goalMx[matchNumber - 1, 1] = Double.NaN;

            for (int j = 0; j < noRounds; j++)
            {
                for (int k = 0; k < NoTeams; k++)
                {
                    sortedGameMx[k, j] = SortedGameMx[k, j];
                }
            }

            return new JagsData
            {
                { "schedule", scheduleMX },
                { "goalsScored", goalMx },
                { "sortedGame", sortedGameMx },
                { "noGames", matchNumber },
                { "noRounds", noRounds},
                { "noTeams", NoTeams },
                { "scale", AttackScaleMx }
            };
        }


        public override JagsData GetSeasonSpecificData()
        {
            for (int i = Constants.SIZE_OF_TRAINING_SET; i < NoMatches; i++)
            {
                GoalMx[i, 0] = Double.NaN;
                GoalMx[i, 1] = Double.NaN;
            }

            // Returns the matrices
            return new JagsData
            {
                { "schedule", ScheduleMx },
                { "goalsScored", GoalMx },
                { "sortedGame", SortedGameMx },
                { "noGames", NoMatches },
                { "noRounds", NoRounds},
                { "noTeams", NoTeams },
                { "scale", AttackScaleMx }
            };
        }
        public override JagsData GetModelPriors()
        {
            return null;
        }
    }
}