﻿using FootballPredictions.DataHandling.JAGSOutputHandlers;
using SharpJags.Jags;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

namespace FootballPredictions.JAGS.DixonColesStatic
{
    /// <summary>
    /// The static Dixon Coles simulation.
    /// </summary>
    public class DixonColesStaticSimulation : SeasonSimulator
    {

        public DixonColesStaticSimulation()
        {
            ModelDefinitionName = "DixonColes";
            ModelDefinitionDefinition =
@"# This is an implementation of  Dixon & Coles (1997)'s  model. 
# Note several tricks:
# 1. To make sure that the sum of alphas (here called 'attack') equal the sum of betas (here called 'defence'), 
#       I have forced both of these vectors to sum to a predefined value (== noTeams). This is OK because I have also introduced  a
#       new goalLambda parameter, which takes care of the offset. It also means that the average of all abilities of a type is 1.
# 2. The awayDisadvantage is a scaler used for the goals scored by the away-team. 
#       It is simply a multiplication constant which scales the no. goals expected by a team when they play away.
#       Even though there have been many examples of analyses where it is shown that the 
#       home/away  (dis-)advantage changes with the team, this is not used by M\{a}her, and thus not by me -- here.
# 3. Dixon&Coles introduce a tau-parameter to make draws more likely in low-scoring games (see paper).
#       This is coded using the ones-trick. The ones-trick requires an upper-bound on the value of this
#       Tau-parameter. It is called C and is declared in the data-section of this file.
# 4. This model only incorporates the STATIC part of the model described in the paper, so is a direct extension of 
#       the M\{a}her model.
#
# HL, 2012-11-24
#

data {
   for (game in 1:noGames) {
        ones[game] <- 1
  }
  C <- 100
}


model {
    goalLambda ~ dgamma(1, 1)
    awayDisadvantage ~ dunif(0, 1)

    # UNscaled abilities
    for (i in 1:noTeams){
       # unscaled abblities
       unscaledDefence[i] ~ dunif(0,1)
       unscaledAttack[i]  ~ dunif(0,1)
    }

    # SCALED abilities
    for (i in 1:noTeams){       
       defence[i] <-  noTeams * unscaledDefence[i] / sum(unscaledDefence)
       attack[i]  <-  noTeams * unscaledAttack[i]  / sum(unscaledAttack)
    }

    # And then, the goal-model
    # Starting to loop over all games
    for (game in 1:noGames){   

        # Home team
	   	lambda[game] <- goalLambda * attack[schedule[1, game]] * defence[ schedule[2, game] ] 
        goalsScored[1, game] ~ dpois( lambda[game] )
	   	
        # Away team
        mu[game]   <- awayDisadvantage * goalLambda * attack[schedule[2, game]] * defence[ schedule[1, game] ] 
   		goalsScored[2, game] ~ dpois( mu[game] )
	
		# The stuff to define the rho
	   	lowerBound[game] <- max(     -1 /  lambda[game], -1 / mu[game] )
	   	upperBound[game] <- min(  1,  1 / (lambda[game]  * mu[game]) )
    	#rho[game] ~ dunif( lowerBound[game], upperBound[game] )


	   	# Declare some variables to find the tau-scaler
	   	is0X[game] <- ifelse(goalsScored[1, game]==0,1,0)
	   	isX0[game] <- ifelse(goalsScored[2, game]==0,1,0)
	   	is1X[game] <- ifelse(goalsScored[1, game]==1,1,0)
	   	isX1[game] <- ifelse(goalsScored[2, game]==1,1,0)
	   	is00[game] <- is0X[game] * isX0[game]
	   	is01[game] <- is0X[game] * isX1[game]
	   	is10[game] <- is1X[game] * isX0[game]
	   	is11[game] <- is1X[game] * isX1[game]

        ones[game] ~ dbern( (is00[game] * ( 1 - lambda[game]*mu[game]*rho ) + 
                            is01[game] * ( 1 + lambda[game]*rho          ) + 
                            is10[game] * ( 1 + mu[game]*rho              ) + 
                            is11[game] * ( 1 - rho                       ) + 
                            1-(is00[game]+is01[game]+is10[game]+is11[game]))/C )
        
    }
    rho ~ dunif( max(lowerBound[]), min(upperBound[]) )
}";
            JagsMonitors.Add(new JagsMonitor()
            {
                ParameterName = "goalsScored",
                Thin = Constants.THINNING
            });
            JagsMonitors.Add(new JagsMonitor()
            {
                ParameterName = "attack",
                Thin = Constants.THINNING
            });
            JagsMonitors.Add(new JagsMonitor()
            {
                ParameterName = "defence",
                Thin = Constants.THINNING
            });
            Initialize(new DixonColesStaticPredictiveModel());
        }
        
        public new void HandleOutput()
        {
            JAGSOutputHandlerStatic handler = new JAGSOutputHandlerStatic(Model);
        }
    }
}