﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

using MathNet.Numerics.LinearAlgebra.Double;
using NHibernate.Linq;
using FootballPredictions.Mappings;
using FootballPredictions.JAGS;
using SharpJags.Jags;
using SharpJags.Math;

namespace FootballPredictions.JAGS.RueSalvesen
{
    /// <summary>
    /// RueSalvesen model that scales the attack and defence strength.
    /// </summary>
    public class DataIntensiveRueSalvesenPredictiveModel : PredictiveModel
    {
        public List<PlayerMatch> PlayerMatches = new List<PlayerMatch>();
        public List<TeamMatch> TeamMatches = new List<TeamMatch>();
        public IDictionary<int, PlayerIndividual> PlayerDictionary = new Dictionary<int, PlayerIndividual>();
        public IDictionary<int, TeamIndividual> TeamDictionary = new Dictionary<int, TeamIndividual>();
        public IDictionary<Match, ProbableLineUp> ProbableHomeLineups = new Dictionary<Match, ProbableLineUp>();
        public IDictionary<Match, ProbableLineUp> ProbableAwayLineups = new Dictionary<Match, ProbableLineUp>();
        public Matrix<double> AttackScaleMx { get; set; }
        public Matrix<double> DefenceScaleMx { get; set; }

        public DataIntensiveRueSalvesenPredictiveModel()
        {
            AssignVariables();
        }

        public override void AssignVariables()
        {
            GetSeasonInformationFromDB();
            InitializeMatrices();
        }

        private void GetSeasonInformationFromDB()
        {
            using (var session = NHibernateConfig.OpenSession())
            {
                //Get matches and teams
                var matches = from g in session.Query<Match>().ToList()
                              where g.Stage.StageId == Constants.STAGE_ID
                              select g;

                foreach (var match in matches)
                {
                    if (!Teams.Contains(match.HomeTeam))
                    {
                        Teams.Add(match.HomeTeam);
                        TeamDictionary.Add(match.HomeTeam.TeamId, new TeamIndividual() { Id = match.HomeTeam.TeamId });
                    }
                    if (!Teams.Contains(match.AwayTeam))
                    {
                        Teams.Add(match.AwayTeam);
                        TeamDictionary.Add(match.AwayTeam.TeamId, new TeamIndividual() { Id = match.AwayTeam.TeamId });
                    }
                }

                matches = matches.OrderBy(m => m.DateAndTime);

                foreach (var match in matches)
                {
                    Matches.Add(match);
                }

                NoTeams = Teams.Count();
                NoMatches = matches.Count();
                NoRounds = (int)(((double)NoMatches / NoTeams) * 2);

                //Get team match information
                var teamMatches = from tm in session.Query<TeamMatch>().ToList()
                                  where tm.Match.Stage.StageId == Constants.STAGE_ID
                                  select tm;

                foreach (var teamMatch in teamMatches.Distinct())
                {
                    TeamMatches.Add(teamMatch);
                }

                //Get Players and player match information
                var playerMatches = from pm in session.Query<PlayerMatch>().ToList()
                                    where pm.TeamMatch.Match.Stage.StageId == Constants.STAGE_ID
                                    select pm;
                var players = from p in session.Query<Player>().ToList()
                              from pm in playerMatches
                              where p.PlayerId == pm.Player.PlayerId
                              select p;
                foreach (var playerMatch in playerMatches.Distinct())
                {
                    PlayerMatches.Add(playerMatch);
                }
                foreach (var player in players.Distinct())
                {
                    PlayerDictionary.Add(player.PlayerId, new PlayerIndividual () { Id = player.PlayerId });
                }

                //Get probable lineups
                var probableLineups = from pl in session.Query<ProbableLineUp>().ToList()
                                      where pl.Match.Stage.StageId == Constants.STAGE_ID
                                      select pl;

                foreach (var probableLineup in probableLineups)
                {
                    if (probableLineup.Team.TeamId == probableLineup.Match.HomeTeam.TeamId)
                    {
                        ProbableHomeLineups.Add(probableLineup.Match, probableLineup);
                    }
                    else if (probableLineup.Team.TeamId == probableLineup.Match.AwayTeam.TeamId)
                    {
                        ProbableAwayLineups.Add(probableLineup.Match, probableLineup);
                    }
                }
            }
        }

        private void InitializeMatrices()
        {
            // Initialize the matrices
            TeamMx = new double[NoMatches, 2];
            GoalMx = new double[NoMatches, 2];
            RoundMx = new double[NoMatches, 2];
            DayMx = new double[NoTeams, NoRounds];
            AttackScaleMx = new double[NoTeams, NoRounds];
            DefenceScaleMx = new double[NoTeams, NoRounds];

            int[] teamGoalsSoFar = new int[NoTeams];
            int[] goalsConcededSoFar = new int[NoTeams];
            List<PlayerMatch> homePlayersInvolved, awayPlayersInvolved;
            TeamMatch homeTeam, awayTeam;
            StartTime = Matches.First().DateAndTime.Ticks;
            var teamMatchesPlayed = new double[NoTeams];
            var matchidx = 0;

            // Fill the matrices here
            foreach (var match in Matches)
            {
                var homeTeamIdx = Teams.IndexOf(match.HomeTeam);
                var awayTeamIdx = Teams.IndexOf(match.AwayTeam);

                teamMatchesPlayed[homeTeamIdx]++;
                teamMatchesPlayed[awayTeamIdx]++;

                TeamMx[matchidx, 0] = homeTeamIdx + 1;
                TeamMx[matchidx, 1] = awayTeamIdx + 1;

                RoundMx[matchidx, 0] = teamMatchesPlayed[homeTeamIdx];
                RoundMx[matchidx, 1] = teamMatchesPlayed[awayTeamIdx];

                DayMx[homeTeamIdx, (int)(teamMatchesPlayed[homeTeamIdx] - 1)] = new TimeSpan(match.DateAndTime.Ticks - StartTime).Days;
                DayMx[awayTeamIdx, (int)(teamMatchesPlayed[awayTeamIdx] - 1)] = new TimeSpan(match.DateAndTime.Ticks - StartTime).Days;

                GoalMx[matchidx, 0] = match.HomeGoals;
                GoalMx[matchidx, 1] = match.AwayGoals;

                homePlayersInvolved = new List<PlayerMatch>();
                awayPlayersInvolved = new List<PlayerMatch>();

                //Get TeamMatch
                homeTeam = TeamMatches.Find(t => t.Match.MatchId == match.MatchId && t.Team.TeamId == match.HomeTeam.TeamId);
                awayTeam = TeamMatches.Find(t => t.Match.MatchId == match.MatchId && t.Team.TeamId == match.AwayTeam.TeamId);

                // Get lineup contributions:
                foreach (var playerMatch in PlayerMatches)
                {
                    if (playerMatch.TeamMatch.Match.MatchId == match.MatchId)
                    {
                        if (playerMatch.TeamMatch.Team.TeamId == match.HomeTeam.TeamId)
                        {
                            homePlayersInvolved.Add(playerMatch);
                        }
                        else
                        {
                            awayPlayersInvolved.Add(playerMatch);
                        }
                    }
                }
                homePlayersInvolved.OrderByDescending(hp => hp.MinsPlayed);
                awayPlayersInvolved.OrderByDescending(ap => ap.MinsPlayed);

                var homeProbableLineupAttackContribution = 0.0;
                var awayProbableLineupAttackContribution = 0.0;
                var homeProbableLineupDefenceContribution = 0.0;
                var awayProbableLineupDefenceContribution = 0.0;

                if (ProbableHomeLineups.ContainsKey(match))
                {
                    homeProbableLineupAttackContribution = CalculateLineupAttackContribution(ProbableHomeLineups[match], matchidx);
                    homeProbableLineupDefenceContribution = CalculateLineupDefenceContribution(ProbableHomeLineups[match], matchidx);
                }
                else
                {
                    var probableLineUp = GetAlterantiveLineUp(homePlayersInvolved, match, match.HomeTeam);
                    homeProbableLineupAttackContribution = CalculateLineupAttackContribution(probableLineUp, matchidx);
                    homeProbableLineupDefenceContribution = CalculateLineupDefenceContribution(probableLineUp, matchidx);
                }

                if (ProbableAwayLineups.ContainsKey(match))
                {
                    awayProbableLineupAttackContribution = CalculateLineupAttackContribution(ProbableAwayLineups[match], matchidx);
                    awayProbableLineupDefenceContribution = CalculateLineupDefenceContribution(ProbableAwayLineups[match], matchidx);
                }
                else
                {
                    var probableLineUp = GetAlterantiveLineUp(awayPlayersInvolved, match, match.AwayTeam);
                    awayProbableLineupAttackContribution = CalculateLineupAttackContribution(probableLineUp, matchidx);
                    awayProbableLineupDefenceContribution = CalculateLineupDefenceContribution(probableLineUp, matchidx);
                }

                var homeTeamTotalAttackContribution = CalculateTeamAttackContribution(TeamDictionary[match.HomeTeam.TeamId]);
                var awayTeamTotalAttackContribution = CalculateTeamAttackContribution(TeamDictionary[match.AwayTeam.TeamId]);
                var homeTeamTotalDefenceContribution = CalculateTeamDefenceContribution(TeamDictionary[match.HomeTeam.TeamId]);
                var awayTeamTotalDefenceContribution = CalculateTeamDefenceContribution(TeamDictionary[match.AwayTeam.TeamId]);
                
                double probableHomeAttackLineupEfficiency = homeTeamTotalAttackContribution == 0.0 ? 1.0 : 
                    homeProbableLineupAttackContribution / homeTeamTotalAttackContribution;
                double probableAwayAttackLineupEfficiency = awayTeamTotalAttackContribution == 0.0 ? 1.0 : 
                    awayProbableLineupAttackContribution / awayTeamTotalAttackContribution;

                double probableHomeDefenceLineupEfficiency = homeTeamTotalDefenceContribution == 0.0 ? 1.0 :
                    homeProbableLineupDefenceContribution / homeTeamTotalDefenceContribution;
                double probableAwayDefenceLineupEfficiency = awayTeamTotalDefenceContribution == 0.0 ? 1.0 :
                    awayProbableLineupDefenceContribution / awayTeamTotalDefenceContribution;

                AttackScaleMx[homeTeamIdx, (int)teamMatchesPlayed[homeTeamIdx] - 1] = probableHomeAttackLineupEfficiency == 0 ? 1 : probableHomeAttackLineupEfficiency;
                AttackScaleMx[awayTeamIdx, (int)teamMatchesPlayed[awayTeamIdx] - 1] = probableAwayAttackLineupEfficiency == 0 ? 1 : probableAwayAttackLineupEfficiency;

                DefenceScaleMx[homeTeamIdx, (int)teamMatchesPlayed[homeTeamIdx] - 1] = probableHomeDefenceLineupEfficiency == 0 ? 1 : probableHomeDefenceLineupEfficiency;
                DefenceScaleMx[awayTeamIdx, (int)teamMatchesPlayed[awayTeamIdx] - 1] = probableAwayDefenceLineupEfficiency == 0 ? 1 : probableAwayDefenceLineupEfficiency;

                if (matchidx > 300)
                {
                }
                //Update info before next round:
                foreach (var playerMatch in homePlayersInvolved)
                {
                    if (playerMatch.MinsPlayed > 0)
                    {
                        UpdatePlayerIndividual(playerMatch);
                    }
                }

                foreach (var playerMatch in awayPlayersInvolved)
                {
                    if (playerMatch.MinsPlayed > 0)
                    {
                        UpdatePlayerIndividual(playerMatch);
                    }
                }

                UpdateTeamIndividual(homeTeam);
                UpdateTeamIndividual(awayTeam);

                matchidx++;
            }
        }

        /// <summary>
        /// Returns attack efficiency of a team.
        /// </summary>
        private double CalculateTeamAttackContribution(TeamIndividual team)
        {
            return (double)team.GoalsScoredSoFar        / Math.Max(1, ((double)team.MinutesPlayedSoFar * 1.4))      * 0.4513
                 + (double)team.ScoringAttSoFar         / Math.Max(1, ((double)team.MinutesPlayedSoFar * 14.11))    * 0.045
                 + (double)team.OnTargetScoringAttSoFar / Math.Max(1, ((double)team.MinutesPlayedSoFar * 4.63))     * 0.135
                 + (double)team.TotalThroughBallSoFar   / Math.Max(1, ((double)team.MinutesPlayedSoFar * 1.7))      * 0.283
                 + (double)team.TotalAttAssist          / Math.Max(1, ((double)team.MinutesPlayedSoFar * 10.52))    * 0.084
                 + (double)team.SuccessfulPassesSoFar   / Math.Max(1, ((double)team.MinutesPlayedSoFar * 367))      * 0.0017;
        }

        /// <summary>
        /// Returns defence efficiency of a team.
        /// </summary>
        private double CalculateTeamDefenceContribution(TeamIndividual team)
        {
            return (double)team.TotalTackle             / Math.Max(1, ((double)team.MinutesPlayedSoFar * 18.39))    * 0.319
                 + (double)team.TotalClearance          / Math.Max(1, ((double)team.MinutesPlayedSoFar * 30.99))    * 0.19
                 + (double)team.Interception            / Math.Max(1, ((double)team.MinutesPlayedSoFar * 15.77))    * 0.373
                 + (double)team.DuelWon                 / Math.Max(1, ((double)team.MinutesPlayedSoFar * 49.69))    * 0.118;
        }

        /// <summary>
        /// Returns attack efficiency of a player.
        /// </summary>
        private double CalculatePlayerAttackContribution(PlayerIndividual player)
        {
            var minsPlayed = player.MinutesPlayedSoFar < 90 ? 90 : player.MinutesPlayedSoFar;
            return (double)player.GoalsScoredSoFar          / (minsPlayed * 1.4) * 0.4513
                 + (double)player.ScoringAttSoFar           / (minsPlayed * 14.11) * 0.045
                 + (double)player.OnTargetScoringAttSoFar   / (minsPlayed * 4.63) * 0.135
                 + (double)player.TotalThroughBallSoFar     / (minsPlayed * 1.7) * 0.283
                 + (double)player.TotalAttAssist            / (minsPlayed * 10.52) * 0.084
                 + (double)player.SuccessfulPassesSoFar     / (minsPlayed * 367) * 0.0017;
        }

        /// <summary>
        /// Returns defence efficiency of a player.
        /// </summary>
        private double CalculatePlayerDefenceContribution(PlayerIndividual player)
        {
            var minsPlayed = player.MinutesPlayedSoFar < 90 ? 90 : player.MinutesPlayedSoFar;
            return (double)player.TotalTackle           / (minsPlayed * 18.39)        * 0.319
                 + (double)player.TotalClearance        / (minsPlayed * 30.99)        * 0.19
                 + (double)player.Interception          / (minsPlayed * 15.77)        * 0.373
                 + (double)player.DuelWon               / (minsPlayed * 49.69)        * 0.118;
        }
        
        /// <summary>
        /// Calculates the attack contribution of the probable lineup.
        /// </summary>
        private double CalculateLineupAttackContribution(ProbableLineUp lineup, int matchidx)
        {
            var lineupContribution = 0.0;
            lineupContribution += CalculatePlayerAttackContribution(PlayerDictionary[ControlPlayerKey(lineup.Player1)]);
            lineupContribution += CalculatePlayerAttackContribution(PlayerDictionary[ControlPlayerKey(lineup.Player2)]);
            lineupContribution += CalculatePlayerAttackContribution(PlayerDictionary[ControlPlayerKey(lineup.Player3)]);
            lineupContribution += CalculatePlayerAttackContribution(PlayerDictionary[ControlPlayerKey(lineup.Player4)]);
            lineupContribution += CalculatePlayerAttackContribution(PlayerDictionary[ControlPlayerKey(lineup.Player5)]);
            lineupContribution += CalculatePlayerAttackContribution(PlayerDictionary[ControlPlayerKey(lineup.Player6)]);
            lineupContribution += CalculatePlayerAttackContribution(PlayerDictionary[ControlPlayerKey(lineup.Player7)]);
            lineupContribution += CalculatePlayerAttackContribution(PlayerDictionary[ControlPlayerKey(lineup.Player8)]);
            lineupContribution += CalculatePlayerAttackContribution(PlayerDictionary[ControlPlayerKey(lineup.Player9)]);
            lineupContribution += CalculatePlayerAttackContribution(PlayerDictionary[ControlPlayerKey(lineup.Player10)]);
            lineupContribution += CalculatePlayerAttackContribution(PlayerDictionary[ControlPlayerKey(lineup.Player11)]);
            return lineupContribution;
        }

        /// <summary>
        /// Calculates the defence contribution of the probable lineup.
        /// </summary>
        private double CalculateLineupDefenceContribution(ProbableLineUp lineup, int matchidx)
        {
            var lineupContribution = 0.0;
            lineupContribution += CalculatePlayerDefenceContribution(PlayerDictionary[ControlPlayerKey(lineup.Player1)]);
            lineupContribution += CalculatePlayerDefenceContribution(PlayerDictionary[ControlPlayerKey(lineup.Player2)]);
            lineupContribution += CalculatePlayerDefenceContribution(PlayerDictionary[ControlPlayerKey(lineup.Player3)]);
            lineupContribution += CalculatePlayerDefenceContribution(PlayerDictionary[ControlPlayerKey(lineup.Player4)]);
            lineupContribution += CalculatePlayerDefenceContribution(PlayerDictionary[ControlPlayerKey(lineup.Player5)]);
            lineupContribution += CalculatePlayerDefenceContribution(PlayerDictionary[ControlPlayerKey(lineup.Player6)]);
            lineupContribution += CalculatePlayerDefenceContribution(PlayerDictionary[ControlPlayerKey(lineup.Player7)]);
            lineupContribution += CalculatePlayerDefenceContribution(PlayerDictionary[ControlPlayerKey(lineup.Player8)]);
            lineupContribution += CalculatePlayerDefenceContribution(PlayerDictionary[ControlPlayerKey(lineup.Player9)]);
            lineupContribution += CalculatePlayerDefenceContribution(PlayerDictionary[ControlPlayerKey(lineup.Player10)]);
            lineupContribution += CalculatePlayerDefenceContribution(PlayerDictionary[ControlPlayerKey(lineup.Player11)]);
            return lineupContribution;
        }

        /// <summary>
        /// Checks if the player exists in the dictionary. If not, the player is added.
        /// </summary>
        private int ControlPlayerKey(Player player)
        {
            if (!PlayerDictionary.ContainsKey(player.PlayerId))
            {
                PlayerDictionary.Add(player.PlayerId, new PlayerIndividual() { Id = player.PlayerId });
            }
            return player.PlayerId;
        }

        /// <summary>
        /// Updates the data stored about a player individual.
        /// </summary>
        private void UpdatePlayerIndividual(PlayerMatch playerMatch)
        {
            PlayerDictionary[playerMatch.Player.PlayerId].GoalsScoredSoFar += playerMatch.Goals;
            PlayerDictionary[playerMatch.Player.PlayerId].ScoringAttSoFar += playerMatch.TotalScoringAtt;
            PlayerDictionary[playerMatch.Player.PlayerId].OnTargetScoringAttSoFar += playerMatch.OntargetScoringAtt;
            PlayerDictionary[playerMatch.Player.PlayerId].PassingAccuracySoFar += playerMatch.AccuratePass / Math.Max(1, playerMatch.TotalPass);
            PlayerDictionary[playerMatch.Player.PlayerId].TotalThroughBallSoFar += playerMatch.TotalThroughBall;
            PlayerDictionary[playerMatch.Player.PlayerId].DribblesSoFar += playerMatch.WonContest;
            PlayerDictionary[playerMatch.Player.PlayerId].TotalAttAssist += playerMatch.TotalAttAssist;
            PlayerDictionary[playerMatch.Player.PlayerId].MinutesPlayedSoFar += playerMatch.MinsPlayed;
            PlayerDictionary[playerMatch.Player.PlayerId].SuccessfulPassesSoFar += playerMatch.AccuratePass;

            PlayerDictionary[playerMatch.Player.PlayerId].TotalTackle += playerMatch.TotalTackle;
            PlayerDictionary[playerMatch.Player.PlayerId].LastManTackle += playerMatch.LastManTackle;
            PlayerDictionary[playerMatch.Player.PlayerId].TotalClearance += playerMatch.TotalClearance;
            PlayerDictionary[playerMatch.Player.PlayerId].SixYardBlock += playerMatch.SixYardBlock;
            PlayerDictionary[playerMatch.Player.PlayerId].Interception += playerMatch.Interception;
            PlayerDictionary[playerMatch.Player.PlayerId].DuelWon += playerMatch.DuelWon;
        }

        /// <summary>
        /// Updates the data stored about a team individual.
        /// </summary>
        private void UpdateTeamIndividual(TeamMatch teamMatch)
        {
            TeamDictionary[teamMatch.Team.TeamId].GoalsScoredSoFar += teamMatch.Goals;
            TeamDictionary[teamMatch.Team.TeamId].ScoringAttSoFar += teamMatch.TotalScoringAtt;
            TeamDictionary[teamMatch.Team.TeamId].OnTargetScoringAttSoFar += teamMatch.OntargetScoringAtt;
            TeamDictionary[teamMatch.Team.TeamId].PassingAccuracySoFar += teamMatch.AccuratePass / Math.Max(1, teamMatch.TotalPass);
            TeamDictionary[teamMatch.Team.TeamId].TotalThroughBallSoFar += teamMatch.TotalThroughBall;
            TeamDictionary[teamMatch.Team.TeamId].DribblesSoFar += teamMatch.WonContest;
            TeamDictionary[teamMatch.Team.TeamId].TotalAttAssist += teamMatch.TotalAttAssist;
            TeamDictionary[teamMatch.Team.TeamId].MinutesPlayedSoFar += 90;
            TeamDictionary[teamMatch.Team.TeamId].SuccessfulPassesSoFar += teamMatch.AccuratePass;
            
            TeamDictionary[teamMatch.Team.TeamId].TotalTackle += teamMatch.TotalTackle;
            TeamDictionary[teamMatch.Team.TeamId].LastManTackle += teamMatch.LastManTackle;
            TeamDictionary[teamMatch.Team.TeamId].TotalClearance += teamMatch.TotalClearance;
            TeamDictionary[teamMatch.Team.TeamId].SixYardBlock += teamMatch.SixYardBlock;
            TeamDictionary[teamMatch.Team.TeamId].Interception += teamMatch.Interception;
            TeamDictionary[teamMatch.Team.TeamId].DuelWon += teamMatch.DuelWon;
        }

        /// <summary>
        /// Used when no probable line up exists for a match.
        /// </summary>
        private ProbableLineUp GetAlterantiveLineUp(List<PlayerMatch> playersInvolved, Match match, Team team)
        {
            return new ProbableLineUp
                    {
                        Player1 = playersInvolved[0].Player,
                        Player2 = playersInvolved[1].Player,
                        Player3 = playersInvolved[2].Player,
                        Player4 = playersInvolved[3].Player,
                        Player5 = playersInvolved[4].Player,
                        Player6 = playersInvolved[5].Player,
                        Player7 = playersInvolved[6].Player,
                        Player8 = playersInvolved[7].Player,
                        Player9 = playersInvolved[8].Player,
                        Player10 = playersInvolved[9].Player,
                        Player11 = playersInvolved[10].Player,
                        Team = team,
                        Match = match
                    };
        }

        /// <summary>
        /// The data required by the season simulator.
        /// </summary>
        public override SharpJags.Jags.JagsData GetMatchSpecificJagsData(int matchNumber)
        {
            var noRounds = 38;

            Matrix<double> goalMx = new double[matchNumber, 2];
            Matrix<double> teamMx = new double[matchNumber, 2];
            Matrix<double> roundMx = new double[matchNumber, 2];
            Matrix<double> dayMx = new double[NoTeams, noRounds];

            for (int i = 0; i < matchNumber; i++)
            {
                goalMx[i, 0] = GoalMx[i, 0];
                goalMx[i, 1] = GoalMx[i, 1];

                teamMx[i, 0] = TeamMx[i, 0];
                teamMx[i, 1] = TeamMx[i, 1];

                roundMx[i, 0] = RoundMx[i, 0];
                roundMx[i, 1] = RoundMx[i, 1];
            }

            goalMx[matchNumber - 1, 0] = Double.NaN;
            goalMx[matchNumber - 1, 1] = Double.NaN;

            for (int j = 0; j < noRounds; j++)
            {
                for (int k = 0; k < NoTeams; k++)
                {
                    dayMx[k, j] = DayMx[k, j];
                }
            }

            return new JagsData
            {
                { "team", teamMx },
                { "round", roundMx },
                { "days", dayMx },
                { "goalsScored", goalMx },
                { "noMatches", matchNumber },
                { "noRounds", noRounds},
                { "noTeams", NoTeams },
                { "ascale", AttackScaleMx },
                { "dscale", DefenceScaleMx },
            };
        }

        /// <summary>
        /// The data required by the season simulator.
        /// </summary>
        public override SharpJags.Jags.JagsData GetSeasonSpecificData()
        {
            for (int i = Constants.SIZE_OF_TRAINING_SET; i < NoMatches; i++)
            {
                GoalMx[i, 0] = Double.NaN;
                GoalMx[i, 1] = Double.NaN;
            }

            // Returns the matrices
            return new JagsData
            {
                { "team", TeamMx },
                { "round", RoundMx },
                { "days", DayMx },
                { "goalsScored", GoalMx },
                { "noMatches", NoMatches },
                { "noRounds", NoRounds},
                { "noTeams", NoTeams },
                { "ascale", AttackScaleMx },
                { "dscale", DefenceScaleMx },
            };
        }

        public override JagsData GetModelPriors()
        {
            return null;
        }
    }
}