﻿using FootballPredictions.DataHandling.JAGSOutputHandlers;
using SharpJags.Jags;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

namespace FootballPredictions.JAGS.RueSalvesen
{
    /// <summary>
    /// Simulation where the mRue Salvesen model is scaled based on the goals scored by the players in the probable starting line-up.
    /// </summary>
    public class GoalScaledRueSalvesenSimulation : SeasonSimulator
    {

        public GoalScaledRueSalvesenSimulation()
        {
            ModelDefinitionName = "IndividualRueSalvesen";
            ModelDefinitionDefinition =
                @"### IndividualRueSalvesen
                #############################################################################################################
                
                data {
                	# Set C^(x) and C^(y) as constants
                	# Values reported by Rue & Salvesen
                    homeGoalAvg <- 0.395
                    awayGoalAvg <- 0.098
                }
                
                model {
                	### Time model
                	#########################################################################################################
                	
                	tau ~ dgamma(10, 1)
                	precision ~ dgamma(10, 0.1)
                	
                	# Loop through all teams and all rounds
                	for(t in 1:noTeams) {
                		# Initial distribution of attack/defence strength
                		unscaledattack[t, 1] 	~ dnorm(0, precision)
                		defence[t, 1] 	~ dnorm(0, precision)
                        attack[t,1] <- unscaledattack[t, 1]
                		
                		# Evolve the attack/defense strength over time with Brownian motion (Wiener process).
                		## The parameters are normally distributed with mean equal to the parameter value of
                		## the previous round. Loss-of-memory effect provided by variance parameter.
                		for(s in 2:noRounds) {
                			
                			unscaledattack[t, s] 	~ dnorm(
                										unscaledattack[t, (s-1)], 
                										(
                											((abs( days[s,t] - days[s-1,t] )) / tau) * precision
                										)
                									)
                			
                			defence[t, s] 	~ dnorm(
                										defence[t, (s-1)], 
                										(
                											((abs( days[s,t] - days[s-1,t] )) / tau) * precision
                										)
                									)
                            attack[t, s] <- unscaledattack[t, s] + log( scale[s, t] )
                		}
                	}
                	
                	### Goal model
                	#########################################################################################################
                	
                	# Params:
                	## attack[t, s] / defence[t, s] = attack/defence strength for team t at given round s
                	## team[i, 1] = home team in game i 
                	## team[i, 2] = away team in game i
                	## round[i, 1] = round(round/games played so far) of home team in game i
                	## round[i, 2] = round(round/games played so far) of away team in game i
                	## goalsScored[i, 1] = goals scored by home team in game i
                	## goalsScored[i, 2] = goals scored by away team in game i
                	
                	# Give the delta-parameter some room to move
                	gamma ~ dunif(0, 0.1)
                	
                	# Loop through all games in correct order
                	for(i in 1:noMatches) {
                		
                		# delta param for psychological effect of underestimating
                		delta[i]			<-	(
                									attack[team[1, i], round[1, i]] + defence[team[1, i], round[1, i]] -
                									attack[team[2, i], round[2, i]] - defence[team[2, i], round[2, i]]
                							) / 2
                							
                		log(homeLambda[i]) 	<- 	(
                									homeGoalAvg + 
                									(
                										attack[team[1, i], round[1, i]] - 
                										defence[team[2, i], round[2, i]] -
                										gamma * delta[i]
                									)
                								)
                		
                		log(awayLambda[i]) 	<- 	(
                									awayGoalAvg + 
                									(
                										attack[team[2, i], round[2, i]] - 
                										defence[team[1, i], round[1, i]] +
                										gamma * delta[i]
                									)
                								)
                		
                        goalsScored[1, i] ~ dpois( homeLambda[i] )
                		goalsScored[2, i] ~ dpois( awayLambda[i] )
                		
                	}	
                }";
            JagsMonitors.Add(new JagsMonitor()
            {
                ParameterName = "goalsScored",
                Thin = Constants.THINNING
            });
            JagsMonitors.Add(new JagsMonitor()
            {
                ParameterName = "attack",
                Thin = Constants.THINNING
            });
            JagsMonitors.Add(new JagsMonitor()
            {
                ParameterName = "defence",
                Thin = Constants.THINNING
            });
            
            Initialize(new GoalScaledRueSalvesenPredictiveModel());
        }
    }
}