﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

using MathNet.Numerics.LinearAlgebra.Double;
using NHibernate.Linq;
using SharpJags.Collections;
using SharpJags.Jags;
using SharpJags.Parsing;

using FootballPredictions.Mappings;
using SharpJags.Math;


namespace FootballPredictions.JAGS.RueSalvesen
{
    /// <summary>
    /// The RueSalvesen prediction model.
    /// </summary>
    public class RueSalvesenPredictiveModel : PredictiveModel
    {
        public RueSalvesenPredictiveModel()
        {
            AssignVariables();
        }

        public override void AssignVariables()
        {
            using (var session = NHibernateConfig.OpenSession())
            {
                using (var tx = session.BeginTransaction())
                {
                    var matches = from g in session.Query<Match>().ToList()
                                  where g.Stage.StageId == Constants.STAGE_ID
                                  select g;

                    foreach (var match in matches)
                    {
                        if (!Teams.Contains(match.HomeTeam))
                        {
                            Teams.Add(match.HomeTeam);
                        }
                        if (!Teams.Contains(match.AwayTeam))
                        {
                            Teams.Add(match.AwayTeam);
                        }
                    }

                    matches = matches.OrderBy(m => m.DateAndTime);

                    foreach (var match in matches)
                    {
                        Matches.Add(match);
                    }

                    NoTeams = Teams.Count();
                    NoMatches = matches.Count();
                    NoRounds = (int)(((double)NoMatches / NoTeams) * 2);
                }

                // Initialize the matrices
                TeamMx = new double[NoMatches, 2];
                GoalMx = new double[NoMatches, 2];
                RoundMx = new double[NoMatches, 2];
                DayMx = new double[NoTeams, NoRounds];

                StartTime = Matches.First().DateAndTime.Ticks;

                var teamMatchesPlayed = new DenseVector(NoTeams, 0);
                var matchidx = 0;

                // Fill the matrices here
                foreach (var match in Matches)
                {
                    var homeTeamIdx = Teams.IndexOf(match.HomeTeam);
                    var awayTeamIdx = Teams.IndexOf(match.AwayTeam);

                    teamMatchesPlayed[homeTeamIdx]++;
                    teamMatchesPlayed[awayTeamIdx]++;

                    // +1 because JAGS has starting index of 1
                    TeamMx[matchidx, 0] = homeTeamIdx + 1;
                    TeamMx[matchidx, 1] = awayTeamIdx + 1;

                    RoundMx[matchidx, 0] = teamMatchesPlayed[homeTeamIdx];
                    RoundMx[matchidx, 1] = teamMatchesPlayed[awayTeamIdx];

                    DayMx[homeTeamIdx, (int)(teamMatchesPlayed[homeTeamIdx] - 1)] = new TimeSpan(match.DateAndTime.Ticks - StartTime).Days;
                    DayMx[awayTeamIdx, (int)(teamMatchesPlayed[awayTeamIdx] - 1)] = new TimeSpan(match.DateAndTime.Ticks - StartTime).Days;

                    GoalMx[matchidx, 0] = match.HomeGoals;
                    GoalMx[matchidx, 1] = match.AwayGoals;
                    
                    matchidx++;
                }
            }
        }

        public override JagsData GetSeasonSpecificData()
        {
            for (int i = Constants.SIZE_OF_TRAINING_SET; i < NoMatches; i++)
            {
                GoalMx[i, 0] = Double.NaN;
                GoalMx[i, 1] = Double.NaN;
            }
            // Returns the matrices
            return new JagsData
            {
                { "team", TeamMx },
                { "round", RoundMx },
                { "days", DayMx },
                { "goalsScored", GoalMx },
                { "noMatches", NoMatches },
                { "noRounds", NoRounds},
                { "noTeams", NoTeams },
            };
        }

        public override JagsData GetMatchSpecificJagsData(int matchNumber)
        {
            //var noRounds = (int)(((double)matchNumber / NoTeams) * 2) + 1;
            var noRounds = 38;

            Matrix<double> goalMx = new double[matchNumber, 2];
            Matrix<double> teamMx = new double[matchNumber, 2];
            Matrix<double> roundMx = new double[matchNumber, 2];
            Matrix<double> dayMx = new double[NoTeams, noRounds];

            for (int i = 0; i < matchNumber; i++)
            {
                goalMx[i, 0] = GoalMx[i, 0];
                goalMx[i, 1] = GoalMx[i, 1];

                teamMx[i, 0] = TeamMx[i, 0];
                teamMx[i, 1] = TeamMx[i, 1];

                roundMx[i, 0] = RoundMx[i, 0];
                roundMx[i, 1] = RoundMx[i, 1];
            }

            goalMx[matchNumber - 1, 0] = Double.NaN;
            goalMx[matchNumber - 1, 1] = Double.NaN;

            for (int j = 0; j < noRounds; j++)
            {
                for (int k = 0; k < NoTeams; k++)
                {
                    dayMx[k, j] = DayMx[k, j];
                }
            }

            return new JagsData
            {
                { "team", teamMx },
                { "round", roundMx },
                { "days", dayMx },
                { "goalsScored", goalMx },
                { "noMatches", matchNumber },
                { "noRounds", noRounds},
                { "noTeams", NoTeams },
            };
        }
        public override JagsData GetModelPriors()
        {
            Matrix<double> attackpriors = new double[NoRounds, NoTeams];
            Matrix<double> defencePriors = new double[NoRounds, NoTeams];
            for(int i = 0; i < NoRounds; i++)
            {
                for (int j = 0; j < NoTeams; j++)
                {
                    attackpriors[i,j] = 1.5;
                    defencePriors[i,j] = 1.5;
                }
            }
            //return new JagsData
            //{
            //    {"attack", attackpriors},
            //    {"defence", defencePriors}
            //};
            return null;
        }
    }
}