﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Web;
using SharpJags.Models;
using SharpJags.Jags;
using SharpJags.Parsing;
using SharpJags.Parsing.Coda;

using FootballPredictions.DataHandling;
using FootballPredictions.DataHandling.JAGSOutputHandlers;
using FootballPredictions.Evaluation.Betting;
using FootballPredictions.Evaluation;
using FootballPredictions.DataHandling.Plots;
using FootballPredictions.Evaluation.Betting.BettingStrategies;

namespace FootballPredictions.JAGS
{
    /// <summary>
    /// Super class for conductiong a simulation. Must be extended by all the simulations.
    /// </summary>
    public abstract class SeasonSimulator
    {
        public string ModelDefinitionName { get; set; }
        public string ModelDefinitionDefinition { get; set; }
        public List<JagsMonitor> JagsMonitors = new List<JagsMonitor>();
        public PredictiveModel Model { get; set; }

              
        public void Initialize(PredictiveModel model)
        {
            JagsConfig.BinPath = Constants.BIN_PATH;
            Model = model;

            var modelDefinition = new ModelDefinition
            {
                Name = ModelDefinitionName,
                Definition = ModelDefinitionDefinition
            };

            var simulation = SimulationSaver.SaveSimulation(model);

            if (!Constants.SEASON_SIMULATION)
            {
                for (int i = Constants.SIMULATION_START_MATCH; i <= Constants.SIMULATION_END_MATCH; i++)
                {
                    var run = new JagsRun
                    {
                        WorkingDirectory = Constants.TEXT_FILES + "JAGS/",
                        ModelDefinition = modelDefinition,
                        ModelPriors = model.GetModelPriors(),
                        ModelData = model.GetMatchSpecificJagsData(i),
                        Monitors = JagsMonitors,
                        Parameters = new MCMCParameters
                        {
                            BurnIn = Constants.BURN_IN,
                            SampleCount = Constants.SAMPLE_COUNT,
                            Chains = Constants.CHAINS
                        },
                    };
                    ISampleParser parser = new ModifiedCodaParser();
                    JagsWrapper.Run(run, parser);
                    JAGSMatchOutputHandler handler = new JAGSMatchOutputHandler(model, i, simulation);
                }  
            }
            else
            {
                var run = new JagsRun
                {
                    WorkingDirectory = Constants.TEXT_FILES + "JAGS/",
                    ModelDefinition = modelDefinition,

                    ModelData = Model.GetSeasonSpecificData(),
                    Monitors = JagsMonitors,
                    Parameters = new MCMCParameters
                    {
                        BurnIn = Constants.BURN_IN,
                        SampleCount = Constants.SAMPLE_COUNT,
                        Chains = Constants.CHAINS
                    },
                };
                ISampleParser parser = new ModifiedCodaParser();
                JagsWrapper.Run(run, parser);
                JAGSSeasonOutputHandler handler = new JAGSSeasonOutputHandler(model, simulation);
            }

            // Betting
            if (Constants.SIMULATE_BETS)
            {
                Betting bet;
                switch (Constants.BETTING_MODEL)
                {
                    case BettingModel.FixedBet:
                        {
                            bet = new FixedBet(simulation.SimulationId, Constants.BOOKMAKER_ID, Constants.SAFETY_MARGIN);
                            break;
                        }
                    case BettingModel.FixedBetNew:
                        {
                            bet = new FixedBetNew(simulation.SimulationId, Constants.BOOKMAKER_ID, Constants.SAFETY_MARGIN);
                            break;
                        }
                    case BettingModel.FixedReturn:
                        {
                            bet = new FixedReturn(simulation.SimulationId, Constants.BOOKMAKER_ID, Constants.SAFETY_MARGIN);
                            break;
                        }
                    case BettingModel.ProbabilityBet:
                        {
                            bet = new ProbabilityBet(simulation.SimulationId, Constants.BOOKMAKER_ID, Constants.SAFETY_MARGIN);
                            break;
                        }
                    case BettingModel.Wallpapering:
                        {
                            bet = new Wallpapering(Constants.SIMULATION_ID_TO_PLOT, Constants.BOOKMAKER_ID, Constants.SAFETY_MARGIN);
                            break;
                        }
                    case BettingModel.Kelly:
                        {
                            bet = new Kelly(Constants.SIMULATION_ID_TO_PLOT, Constants.BOOKMAKER_ID, Constants.SAFETY_MARGIN);
                            break;
                        }
                    default:
                        { 
                            break; 
                        }
                }
            }

            // RPS
            if (Constants.GENERATE_RPS)
            {
                new GenerateRPS(simulation);
            }
        }
    }
}